function formulaserializer(isFlow){
	this.independentFunctions = {'NOW':'ModifiedAt', 'TODAY':'ModifiedAt', 'RAND':'ModifiedAt', 'PI':'CreatedAt'};

	this.getList = function(instance, refCellName, childModelName){
		if (isFlow)
			return instance.getList(refCellName, childModelName)
		else
			return instance.getList(refCellName+"."+childModelName)
	}
	this.getFormula = function(formula){
		if(!formula)
			return ''
		node = this.getList(formula, 'Formula', 'Node')[0]
		return encodeToAscii(this.constructFormulaStr(node));
	}
	this.constructFormulaStr = function(node){
		return this.getArgument(node)
	}
	this.getArgument = function(node){
		if (!node){
			return '#REF!'
		}else if (node.getValue('IsSyntacticSugar')){
			return this.resolveSyntacticSugar(node)
        }else if (node.getValue('NodeType') == 'Function' || node.getValue('NodeType') == 'Fetch3DFunction' || node.getValue('NodeType') == 'Aggrigate3DFunction'){
			var nodeList = this.getList(node, 'Node', 'Node')
			var argStr = ''
			var argStack = new Array()
			sortEntities(nodeList,'ArgNo')
			if(!this.independentFunctions[node.getValue('Value')]){
				for(nodeId in nodeList){
					var arg = this.getArgument(nodeList[nodeId])
					argStr = argStr+arg+','
					argStack.push(arg)
				}
			}
			argStr = argStr.substring(0, argStr.length-1)
			return this.addParenthesisSpace(node, this.getFunctionString(node, argStr, argStack))
		}else if (node.getValue('NodeType') == 'Constant'){
			if(node.getValue('Value'))
				if(node.getValue('IsStr')>=1.0)
					return this.addParenthesisSpace(node,'"'+node.getValue('Value').toString()+'"')
				else
					return this.addParenthesisSpace(node,node.getValue('Value').toString())
			return node.getValue('IsStr')>=1.0?this.addParenthesisSpace(node,'""'):this.addParenthesisSpace(node,'0')
		}else if (node.getValue('NodeType') == 'CellReference'){
            return this.addParenthesisSpace(node, this.resolveCellParameter(node));
		}else if (node.getValue('NodeType') == '3dCellReference'){
			if(!node.get('CellMetadata'))
				return this.addParenthesisSpace(node,'#REF!')
			if(node.get('CellMetadata').getValue('CellName')){
				if(!node.get('CellMetadata').get('SheetMetadata'))
					return this.addParenthesisSpace(node,'#REF!')
				return this.addParenthesisSpace(node,node.get('CellMetadata').get('SheetMetadata').getValue('SheetName')+'.'+node.get('CellMetadata').getValue('CellName'))
            }
			else
				return this.addParenthesisSpace(node,'#REF!')
		}
	}
	this.resolveSyntacticSugar = function(node){
		if (node.getValue('NodeType') == 'Fetch3DFunction'){
			return this.resolveDGETSugarString(node)
		}else if (node.getValue('NodeType') == 'Aggrigate3DFunction'){
			return this.resolveAggSugarString(node)
		}else if (node.getValue('NodeType') == 'Function' && node.getValue('Value') == 'FORMAT'){
			return this.resolveTemplateSugarString(node)
		}
	}
	this.resolveTemplateSugarString = function(node){
		var nodeList = this.getList(node, 'Node', 'Node')
		sortEntities(nodeList, 'ArgNo');
		var argStack = new Array();
		for(nodeId in nodeList){
			var arg = this.getArgument(nodeList[nodeId]);
			argStack.push(arg);
		}
		var templStr = argStack.shift();
		templStr = templStr.replace(/\{(\d+)\}/g, function(a, digits) {return '{'+argStack[digits]+'}'})
		return this.addParenthesisSpace(node, 'TEMPLATE('+templStr+')');
	}
    this.resolveCellParameter = function(node){
        if(!node.get('CellMetadata')){
            if(isFlow && node.getDisplayValue('CellMetadata')){
                var dispToken = node.getDisplayValue('CellMetadata').split('-');
                return dispToken[dispToken.length-1];
            }
            return '#REF!';
        }
        if(node.getValue('IsPublicRef') && node.getValue('IsPublicRef')==1)
            if(node.get('CellMetadata').getValue('CellName'))
                return node.get('CellMetadata').getValue('CellName');
            else
                return node.get('CellMetadata').getValue('CellDisplayName');
        return node.get('CellMetadata').getValue('CellDisplayName');
    }
	this.resolveDGETSugarString = function(node){
		var nodeList = this.getList(node, 'Node', 'Node')
        sortEntities(nodeList,'ArgNo');
        var refCellName = this.resolveCellParameter(nodeList[0]);
        var fetchCellName = this.resolveCellParameter(nodeList[1]);
        return this.addParenthesisSpace(node, refCellName+'.'+fetchCellName)
	}

	this.resolveAggSugarString = function(node){
		var nodeList = this.getList(node, 'Node', 'Node');
		var argStr = ''
		var argStack = new Array()
		sortEntities(nodeList,'ArgNo')
        
        if(node.getValue('IsSyntacticSugar')==2 && isFlow)
            var arg = nodeList[0].get('CellMetadata').get('SheetMetadata').getValue('PublicName').replace(/ /g, '_')+'.'+nodeList[1].get('CellMetadata').getValue('CellName')
        else
            var arg = nodeList[0].get('CellMetadata').get('SheetMetadata').getValue('SheetName')+'['+nodeList[0].get('CellMetadata').getValue('CellName')+'].'+nodeList[1].get('CellMetadata').getValue('CellName')
        
		argStr = argStr+arg+','
		argStack.push(arg)

        var cArgs = nodeList.slice(2, 10)
		for(nodeId in cArgs){
			arg = this.getArgument(cArgs[nodeId])
			argStr = argStr+arg+','
			argStack.push(arg)
		}
		argStr = argStr.substring(0, argStr.length-1)
		return this.addParenthesisSpace(node, this.getFunctionString(node, argStr, argStack))
	}

	this.getFunctionString = function(node, argStr, argStack){
        var value = node.getValue('Value')
		operator = this.operators[value]
		if(operator){
			if(argStack.length==2){
				return argStack[0]+operator+argStack[1]
			}else if(argStack.length==1){
				return argStack[0]+operator
			}
		}
		return value +'('+argStr+')'
	}
	this.operators = {'addOper':'+', 'subOper':'-', 'mulOper':'*', 'divOper':'/', 'eqOper':'=', 'ltOper':'<', 'gtOper':'>', 'gtEqOper':'>=', 'notEqOper':'<>', 'ltEqOper':'<=', 'percentageOper':'%', 'powerOper':'^'}

	this.addParenthesisSpace = function(node,nodeText)
	{
		if(node.getValue('LeadingWS'))
			for(i=0;i<node.getValue('LeadingWS');i++)
				nodeText = ' '+nodeText;
		if(node.getValue('TrailingWS'))
			for(i=0;i<node.getValue('TrailingWS');i++)
				nodeText = nodeText+' ';
		if(node.getValue('IsParenthesised') && node.getValue('IsParenthesised')==1)
			nodeText = '('+nodeText+')'
		return nodeText



	}

}