import appconfig as config
from routes.mapper import Mapper
from webob import Request
import logging

def isWebService(environ, result):
        req = Request(environ)
        if(req.params.getall("wsdl")):
            return True
        else:
            return False

def referals(environ, result):
        req = Request(environ)
        if(req.params.get('action', False)):
            if(req.method=='PUT' and req.params['action']=='submit'):
                return True
        else:
            return False

def isAttachment(environ, result):
        req = Request(environ)
        url=req.path_info
        path=url[1:len(url)].split('/')
        if(path[len(path)-1]=='attachment'):
            return True
        else:
            return False

def isCheckout(environ, result):
        req = Request(environ)
        url=req.path_info
        path=url[1:len(url)].split('/')
        return (path[len(path)-1]=='checkout') or (path[len(path)-1]=='commit')

def isCsvimport(environ, result):
        req = Request(environ)
        url=req.path_info
        path=url[1:len(url)].split('/')
        return path[len(path)-1]=='csvimport'

def isSerializeApplication(environ, result):
    req = Request(environ)
    url=req.path_info
    path=url[1:len(url)].split('/')
    return (path[len(path)-1]=='application')

def isInstance(environ, result):
        req = Request(environ)
        url=req.path_info
        path=url[1:len(url)].split('/')

        if(path[len(path)-1]=='list'):
            return False
        else:
            return True

def iscustomfunction(environ, result):
        req = Request(environ)
        if(req.params.get('action', False)):
            if(req.params['action']=='invoke'):
                return True
            else:
                return False
        else:
            return False

def getInstanceController():

    if config.env==config.ENTERPRISE:
        return 'orangescape.model.server.commands.ECommandFactory:ECommandFactory'
    elif config.env in (config.TEST, config.GAE, config.GAESQL):
        return 'orangescape.model.server.commands.GAECommandFactory:GAECommandFactory'
    elif config.env== config.AZURE:
        return 'orangescape.model.server.commands.AzureCommandFactory:AzureCommandFactory'
    elif config.env== config.SQL:
        return 'orangescape.model.server.commands.SQLCommandFactory:SQLCommandFactory'
    elif config.env== config.SL:
        return 'SLCommandFactory:SLCommandFactory'

def getDataViewer():
    return 'orangescape.model.server.DataViewer:DataViewer'

def make_map(instanceController = None):

        if instanceController is None:
            instanceController = getInstanceController()

        map = Mapper(explicit = True, register = False)
        map.connect('/{applicationid}/{metadatatype}/administrator', controller=getDataViewer(), action='getDetails')
        map.connect('/{applicationid}/{metadatatype}/administrator/*url', controller=getDataViewer() , action='getData')

        """Flow service - apis"""
        map.connect('/api/{version}/*url', controller=instanceController, action='apiHandler')

        """Flow Purchase"""
        map.connect('/{applicationid}/{metadatatype}/{metadataid}/{activityid}/purchase', controller=instanceController, action='purchase',section='open', conditions=dict(method=['POST']))

        """studio based mapping"""
        map.connect('/{applicationid}/{metadatatype}/{parentsheetmetadataid}/{taskid}/{sheetmetadataid}/*url/viewversions', controller=instanceController, action='modelversions')
        map.connect('/{applicationid}/{metadatatype}/{parentsheetmetadataid}/{taskid}/{sheetmetadataid}/*url/downloadversion', controller=instanceController, action='downloadversion')
        map.connect('/{applicationid}/{metadatatype}/{parentsheetmetadataid}/{taskid}/{sheetmetadataid}/*url/application', controller=instanceController, action='serializeApplication', section='open', conditions=dict(method=['GET'],function=isSerializeApplication))
        map.connect('/{applicationid}/{metadatatype}/{parentsheetmetadataid}/{taskid}/{sheetmetadataid}/*url/replaceattachment', controller=instanceController, action='replaceAttachment', conditions=dict(method=['PUT']))
        map.connect('/{applicationid}/{metadatatype}/{parentsheetmetadataid}/{taskid}/{sheetmetadataid}/*url/replaceallattachment', controller=instanceController, action='replaceAllAttachment', conditions=dict(method=['PUT']))
        map.connect('/{applicationid}/{metadatatype}/{parentsheetmetadataid}/{taskid}/{sheetmetadataid}/*url/formdata', controller=instanceController, action='serializeFormData')

        map.connect('/migrate/clearCache', controller='orangescape.model.server.Repair:Migrate', action='clearCache',section='open')
        map.connect('/migrate/updateAppRef', controller='orangescape.model.server.Repair:Migrate', action='updateAppRef')
        map.connect('/migrate/updateAppSubscription', controller='orangescape.model.server.Repair:Migrate', action='updateAppSubscription')
        map.connect('/migrate/unlockModel', controller='orangescape.model.server.Repair:Migrate', action='unlockModel')


        """runtime flatimport,sla,cache clearing mapping"""
        map.connect('/{applicationid}/{metadatatype}/flatcsvimport', controller='orangescape.model.server.integration.FlatCsvImporter:FlatCsvImport', action='post', section='open')
        map.connect('/{applicationid}/{metadatatype}/triggersla', controller=instanceController, action='triggersla', section='open')
        map.connect('/clearcache', controller=instanceController, action='clearCache',section='open')

        """webserice based mapping"""
        map.connect('/{applicationid}/{metadatatype}/{parentsheetmetadataid}/{taskid}/*url', controller=instanceController, action='wsdl', section='open', conditions=dict(function=isWebService))
        map.connect('/{applicationid}/{metadatatype}/{parentsheetmetadataid}/{taskid}/{sheetmetadataid}/ws', controller=instanceController, action='invokeWS', section='open', conditions=dict(method=['POST']))

        map.connect('/{applicationid}/{metadatatype}/{sheetmetadataid}/query', controller=instanceController, action='queryByTable', section='admin', conditions=dict(method=['GET']))

        """Mapping for report sync api"""
        map.connect('/{applicationid}/{metadatatype}/{sheetmetadataid}/{activitydefinitionid}/keys', controller=instanceController, action='keysByActivity', section='admin', conditions=dict(method=['GET']))
        map.connect('/{applicationid}/{metadatatype}/{sheetmetadataid}/{activitydefinitionid}/query', controller=instanceController, action='queryByActivity', section='admin', conditions=dict(method=['GET']))
        map.connect('/{applicationid}/{metadatatype}/{sheetmetadataid}/{activitydefinitionid}/query', controller=instanceController, action='getbykeysByActivity', section='admin', conditions=dict(method=['POST']))
        map.connect('/{applicationid}/{metadatatype}/{sheetmetadataid}/{activitydefinitionid}/aggregate', controller=instanceController, action='aggregateEntitiesByActivity', conditions=dict(method=['GET']))

        map.connect('/{applicationid}/{metadatatype}/{sheetmetadataid}/{activitydefinitionid}/keys/:secondaryuser', controller=instanceController, action='keysByActivity', section='admin', conditions=dict(method=['GET']))
        map.connect('/{applicationid}/{metadatatype}/{sheetmetadataid}/{activitydefinitionid}/query/:secondaryuser', controller=instanceController, action='queryByActivity', section='admin', conditions=dict(method=['GET']))
        map.connect('/{applicationid}/{metadatatype}/{sheetmetadataid}/{activitydefinitionid}/query/:secondaryuser', controller=instanceController, action='getbykeysByActivity', section='admin', conditions=dict(method=['POST']))
        map.connect('/{applicationid}/{metadatatype}/{sheetmetadataid}/{activitydefinitionid}/aggregate/:secondaryuser', controller=instanceController, action='aggregateEntitiesByActivity', section='admin', conditions=dict(method=['GET']))

        map.connect('/{applicationid}/{metadatatype}/{parentsheetmetadataid}/{taskid}/{sheetmetadataid}/{instanceId}/{cellName}/keys', controller=instanceController, action='keys', section='admin', conditions=dict(method=['GET']))
        map.connect('/{applicationid}/{metadatatype}/{parentsheetmetadataid}/{taskid}/{sheetmetadataid}/{instanceId}/{cellName}/query', controller=instanceController, action='query', section='admin', conditions=dict(method=['GET']))
        map.connect('/{applicationid}/{metadatatype}/{parentsheetmetadataid}/{taskid}/{sheetmetadataid}/{instanceId}/{cellName}/query', controller=instanceController, action='getbykeys', section='admin', conditions=dict(method=['POST']))
        map.connect('/{applicationid}/{metadatatype}/{parentsheetmetadataid}/{taskid}/{sheetmetadataid}/{instanceId}/{cellName}/aggregate', controller=instanceController, action='aggregate', conditions=dict(method=['GET']))

        map.connect('/{applicationid}/{metadatatype}/{sheetmetadataid}/report/{searchdefinitionid}/keys', controller=instanceController, action='keysBySearch', section='admin', conditions=dict(method=['GET']))
        map.connect('/{applicationid}/{metadatatype}/{sheetmetadataid}/report/{searchdefinitionid}/query', controller=instanceController, action='queryBySearch', section='admin', conditions=dict(method=['GET']))
        map.connect('/{applicationid}/{metadatatype}/{sheetmetadataid}/report/{searchdefinitionid}/query', controller=instanceController, action='getbykeysBySearch', section='admin', conditions=dict(method=['POST']))
        map.connect('/{applicationid}/{metadatatype}/{sheetmetadataid}/report/{searchdefinitionid}/meta', controller=instanceController, action='getMetaInfo', conditions=dict(method=['GET']))
        map.connect('/{applicationid}/{metadatatype}/{sheetmetadataid}/report/{searchdefinitionid}/aggregate', controller=instanceController, action='aggregateEntitiesBySearch', conditions=dict(method=['GET']))
        map.connect('/{applicationid}/{metadatatype}/{sheetmetadataid}/report/{searchdefinitionid}/{mastersearchid}',controller=instanceController, action='readforMasterReport', conditions=dict(method=['GET']))

        """Mapping for normal Report"""
        map.connect('/{applicationid}/{metadatatype}/{sheetmetadataid}/report/{searchdefinitionid}', controller=instanceController, action='readforReport', conditions=dict(method=['GET']))
        map.connect('/{applicationid}/{metadatatype}/{sheetmetadataid}/report/{searchdefinitionid}/p{pageNumber}/{pageCount}', controller=instanceController, action='readforReport', conditions=dict(method=['GET']))
        map.connect('/{applicationid}/{metadatatype}/{sheetmetadataid}/report/{searchdefinitionid}/p{pageNumber}/{pageCount}', controller=instanceController, action='invokeforReport',conditions=dict(function=iscustomfunction))
        map.connect('/{applicationid}/{metadatatype}/{parentsheetmetadataid}/report/{searchdefinitionid}/{sheetmetadataid}/{sheetid}', controller=instanceController, action='readforInstanceReport', conditions=dict(method=['GET']))
        map.connect('/{applicationid}/{metadatatype}/{parentsheetmetadataid}/report/{searchdefinitionid}/{sheetmetadataid}/{sheetid}', controller=instanceController, action='invokeforInstanceReport',conditions=dict(function=iscustomfunction))

        """Mapping for Inbox """
        map.connect('/{applicationid}/{metadatatype}/{sheetmetadataid}/{activitydefinitionid}', controller=instanceController, action='readforInbox', conditions=dict(method=['GET']))
        map.connect('/{applicationid}/{metadatatype}/{sheetmetadataid}/{activitydefinitionid}/p{pageNumber}', controller=instanceController, action='readforInbox', conditions=dict(method=['GET']))
        map.connect('/{applicationid}/{metadatatype}/{sheetmetadataid}/{activitydefinitionid}/p{pageNumber}/{pageCount}', controller=instanceController, action='readforInbox', conditions=dict(method=['GET']))
        map.connect('/{applicationid}/{metadatatype}/{sheetmetadataid}/{activitydefinitionid}/p{pageNumber}/{pageCount}', controller=instanceController, action='updateUser', conditions=dict(method=['PUT']))
        map.connect('/{applicationid}/{metadatatype}/{sheetmetadataid}/{activitydefinitionid}/p{pageNumber}/{pageCount}/*url', controller=instanceController, action='deleteforInbox', conditions=dict(method=['DELETE'],function=isInstance))
        map.connect('/{applicationid}/{metadatatype}/{sheetmetadataid}/{activitydefinitionid}/p{pageNumber}/{pageCount}', controller=instanceController, action='invokeforInbox', conditions=dict(function=iscustomfunction))
        map.connect('/{applicationid}/{metadatatype}/{sheetmetadataid}/{activitydefinitionid}/p{pageNumber}/{pageCount}/*url', controller=instanceController, action='invokeforInbox', conditions=dict(function=iscustomfunction))
        map.connect('/{applicationid}/{metadatatype}/{sheetmetadataid}/{activitydefinitionid}/p{pageNumber}/{pageCount}/{cellName}/search', controller=instanceController, action='searchUser', conditions=dict(method=['GET']))

        #for secondary user
        map.connect('/{applicationid}/{metadatatype}/{sheetmetadataid}/{activitydefinitionid}/:SecondaryUser', controller=instanceController, action='readforInbox', conditions=dict(method=['GET']))
        map.connect('/{applicationid}/{metadatatype}/{sheetmetadataid}/{activitydefinitionid}/:SecondaryUser/p{pageNumber}/{pageCount}', controller=instanceController, action='readforInbox', conditions=dict(method=['GET']))
        map.connect('/{applicationid}/{metadatatype}/{sheetmetadataid}/{activitydefinitionid}/:SecondaryUser/p{pageNumber}/{pageCount}', controller=instanceController, action='updateUser', conditions=dict(method=['PUT']))
        map.connect('/{applicationid}/{metadatatype}/{sheetmetadataid}/{activitydefinitionid}/:SecondaryUser/p{pageNumber}/{pageCount}/*url', controller=instanceController, action='deleteforInbox', conditions=dict(method=['DELETE'],function=isInstance))
        map.connect('/{applicationid}/{metadatatype}/{sheetmetadataid}/{activitydefinitionid}/:SecondaryUser/p{pageNumber}/{pageCount}', controller=instanceController, action='invokeforInbox', conditions=dict(function=iscustomfunction))
        map.connect('/{applicationid}/{metadatatype}/{sheetmetadataid}/{activitydefinitionid}/:SecondaryUser/p{pageNumber}/{pageCount}/*url', controller=instanceController, action='invokeforInbox', conditions=dict(function=iscustomfunction))
        map.connect('/{applicationid}/{metadatatype}/{sheetmetadataid}/{activitydefinitionid}/:SecondaryUser/p{pageNumber}/{pageCount}/{cellName}/search', controller=instanceController, action='searchUser', conditions=dict(method=['GET']))

        map.connect('/{applicationid}/{metadatatype}/{parentsheetmetadataid}/{taskid}/{sheetmetadataid}/{instanceId}/{cellName}/search', controller=instanceController, action='readforSearch', conditions=dict(method=['GET']))
        map.connect('/{applicationid}/{metadatatype}/{parentsheetmetadataid}/{taskid}/{sheetmetadataid}/{instanceId}/{cellName}/search/p{pageNumber}/{pageCount}', controller=instanceController, action='readforSearch', conditions=dict(method=['GET']))
        map.connect('/{applicationid}/{metadatatype}/{parentsheetmetadataid}/{taskid}/{sheetmetadataid}/{instanceId}/*url/{cellName}/search', controller=instanceController, action='readforSearch', conditions=dict(method=['GET']))
        #added section public for public lookup from public forms
        map.connect('/{applicationid}/{metadatatype}/{parentsheetmetadataid}/{taskid}/{sheetmetadataid}/{instanceId}/*url/{cellName}/search/p{pageNumber}/{pageCount}', controller=instanceController, action='readforSearch', conditions=dict(method=['GET']), section='public')

        #added section public for lookup meta from public forms
        map.connect('/{applicationid}/{metadatatype}/{parentsheetmetadataid}/{taskid}/*url/meta', controller=instanceController, action='metaInfoByModel', conditions=dict(method=['GET']), section='public')

        """attachment based mapping"""
        map.connect('/{applicationid}/{metadatatype}/{parentsheetmetadataid}/{taskid}/{sheetmetadataid}/*url/attachment', controller=instanceController, action='getAttachmentWithId', conditions=dict(method=['GET'],function=isAttachment))
        map.connect('/{applicationid}/{metadatatype}/{parentsheetmetadataid}/{taskid}/{sheetmetadataid}/*url/attachment', controller=instanceController, action='updateAttachmentWithId', conditions=dict(method=['POST'],function=isAttachment))
        map.connect('/{applicationid}/{metadatatype}/{parentsheetmetadataid}/{taskid}/{sheetmetadataid}/*url/attachment', controller=instanceController, action='deleteAttachmentWithId', conditions=dict(method=['DELETE'],function=isAttachment))
        """attachment for bulk mapping"""
        map.connect('/{applicationid}/{metadatatype}/{parentsheetmetadataid}/{taskid}/*url/bulkattach', controller=instanceController, action='attach', conditions=dict(method=['GET']))
        map.connect('/{applicationid}/{metadatatype}/{parentsheetmetadataid}/{taskid}/{sheetmetadataid}/*url/attachment', controller=instanceController, action='updateAttachment', conditions=dict(method=['PUT'],function=isAttachment))
        """attachment unused mapping"""
        map.connect('/{applicationid}/{metadatatype}/{parentsheetmetadataid}/{taskid}/*url/attach', controller=instanceController, action='attachWithId', conditions=dict(method=['GET']))
        map.connect('/{applicationid}/{metadatatype}/{parentsheetmetadataid}/{taskid}/{sheetmetadataid}/*url/getattachment', controller=instanceController, action='getAttachment', conditions=dict(method=['GET']))

        """Flow service - public forms, invoke is made public"""
        map.connect('/{applicationid}/{metadatatype}/{parentsheetmetadataid}/{taskid}/{sheetmetadataid}/{instanceId}/public', controller=instanceController, action='publicInvoke',conditions=dict(function=iscustomfunction), section='public')
        map.connect('/{applicationid}/{metadatatype}/{parentsheetmetadataid}/{taskid}/{sheetmetadataid}/{instanceId}/public/*url', controller=instanceController, action='publicInvoke',conditions=dict(function=iscustomfunction), section='public')

        """editor based mapping"""
        map.connect('/{applicationid}/{metadatatype}/{parentsheetmetadataid}/{taskid}/{sheetmetadataid}', controller=instanceController, action='new',conditions=dict(method=['POST']))
        map.connect('/{applicationid}/{metadatatype}/{parentsheetmetadataid}/{taskid}/{sheetmetadataid}/*url', controller=instanceController, action='invoke',conditions=dict(function=iscustomfunction))

        """studio based mapping"""
        map.connect('/{applicationid}/{metadatatype}/{parentsheetmetadataid}/{taskid}/{sheetmetadataid}/*url/commit', controller=instanceController, action='commit', conditions=dict(method=['POST'],function=isCheckout))

        """editor based mapping"""
        map.connect('/{applicationid}/{metadatatype}/{parentsheetmetadataid}/{taskid}/{sheetmetadataid}/*url/csvimport', controller=instanceController, action='csvimport', conditions=dict(method=['POST'],function=isCsvimport))
        map.connect('/{applicationid}/{metadatatype}/{parentsheetmetadataid}/{taskid}/{sheetmetadataid}/*url', controller=instanceController, action='create', conditions=dict(method=['POST']))

        """studio based mappig"""
        map.connect('/{applicationid}/{metadatatype}/{parentsheetmetadataid}/{taskid}/{sheetmetadataid}/*url/checkout', controller=instanceController, action='checkout', section='open', conditions=dict(method=['GET'],function=isCheckout))

        """for deleting attachment """
        map.connect('/{applicationid}/{metadatatype}/{parentsheetmetadataid}/{taskid}/{sheetmetadataid}/*url/attachment', controller=instanceController, action='deleteAttachment', conditions=dict(method=['DELETE'],function=isAttachment))

        """editor based mapping"""
        map.connect('/{applicationid}/{metadatatype}/{parentsheetmetadataid}/{taskid}/{sheetmetadataid}/*url', controller=instanceController, action='read', conditions=dict(method=['GET'],function=isInstance))
        map.connect('/{applicationid}/{metadatatype}/{parentsheetmetadataid}/{taskid}/{sheetmetadataid}/*url', controller=instanceController, action='update', conditions=dict(method=['PUT'],function=isInstance))
        map.connect('/{applicationid}/{metadatatype}/{parentsheetmetadataid}/{taskid}/{sheetmetadataid}/*url', controller=instanceController, action='deleteAction', conditions=dict(method=['DELETE'],function=isInstance))

        """studio loading and creating models"""
        map.connect('/studio', controller=instanceController, action='openModel')
        map.connect('/reload', controller=instanceController, action='reloadModel', conditions=dict(method=['PUT']))
        map.connect('{applicationid}/{metadatatype}/sheetimport', controller=instanceController, action='importModel', conditions=dict(method=['POST']))
        """studio theme uplaod"""
        map.connect('/themeUpload', controller=instanceController, action='themeUpload', conditions=dict(method=['POST']))

        """login,home page mapping"""
        map.connect('/{applicationid}/{metadatatype}/signin', controller=instanceController, action='processLogin',section='open', conditions=dict(method=['POST']))
        map.connect('/{applicationid}/{metadatatype}/signin', controller=instanceController, action='renderLogin', section='open', conditions=dict(method=['GET']))
        map.connect('/{applicationid}/{metadatatype}/schemasync', controller=instanceController, action='schemaSync', section='open', conditions=dict(method=['GET']))
        map.connect('/{applicationid}/{metadatatype}/metadatasync', controller=instanceController, action='metadataSync', section='open', conditions=dict(method=['GET']))
        map.connect('/{applicationid}/{metadatatype}/usersync', controller=instanceController, action='userSync', conditions=dict(method=['GET']))
        map.connect('/{applicationid}/{metadatatype}/authenticate', controller=instanceController, action='authenticate', section='open')
        map.connect('/{applicationid}/{metadatatype}/menu', controller=instanceController, action='menu')
        map.connect('/{applicationid}/{metadatatype}/worktop', controller=instanceController, action='renderWorktop')
        map.connect('/{applicationid}/{metadatatype}/teamworktop', controller=instanceController, action='renderWorktop')
        map.connect('/{applicationid}/{metadatatype}/home', controller=instanceController, action='renderHome')
        map.connect('/{applicationid}/{metadatatype}/template', controller=instanceController, action='customTemplate')
        map.connect('/', controller=instanceController, action='renderHome')
        map.connect('/_ah/login_required', controller=instanceController, action='renderLogin',section='open', conditions=dict(method=['GET']))
        map.connect('{applicationid}/{metadatatype}/logout', controller=instanceController, action='logout')


        """Flow services"""
        map.connect('{applicationid}/{metadatatype}/flowSetup', controller=instanceController, action='flowSetup', conditions=dict(method=['POST']))
        map.connect('{applicationid}/{metadatatype}/initApp', controller=instanceController, action='initApp', conditions=dict(method=['POST']))
        map.connect('/auth', controller=instanceController, action='authorize', section='open')
        map.connect('/attachmentList', controller=instanceController, action='getDocList')
        map.connect('/{applicationid}/{metadatatype}/ping', controller=instanceController, action='ping', section='open')
        map.connect('/exportdata', controller=instanceController, action='exportdata' ,section='open')
        map.connect('/masterexport', controller=instanceController, action='masterDataExport' ,section='open')
        map.connect('/masterimport', controller=instanceController, action='masterDataImport' ,section='open')
        map.connect('/masterbatchimport', controller=instanceController, action='masterBatchImport' ,section='open')
        map.connect('/reportexport', controller=instanceController, action='reportExport' ,section='open')
        map.connect('/tq', controller=instanceController, action='taskQueueHandler' ,section='open')
        map.connect('/kpireportexport', controller=instanceController, action='kpireportExport' ,section='open')
        map.connect('/readMemcache', controller=instanceController, action='readMemcache' ,section='open')
        """Flow service - public forms"""
        map.connect('{sheetmetadataid}/public/create', controller=instanceController, action='publicCreate', conditions=dict(method=['POST']), section='public')
        map.connect('{sheetmetadataid}/public/create', controller=instanceController, action='renderPublicCreate', section='public')
        map.connect('{sheetmetadataid}/{instanceId}/public/submit', controller=instanceController, action='publicSubmit', conditions=dict(method=['POST']), section='public')

        """incoming email mapping"""
        map.connect('/_ah/mail/*url', controller=instanceController, action='receiveMail',section='open', conditions=dict(method=['POST']))

        """For flow instace warm_up for performance."""
        map.connect('/_ah/warmup', controller=instanceController, action='ping',section='open')

        """backend Start"""
        map.connect('/_ah/start', controller=instanceController, action='backendStart',section='open', conditions=dict(method=['GET']))

        """testcases based mapping"""
        map.connect('/testjstl/{sheetmetadataid}/{instanceId}', controller='orangescape.tests.config.TestRouting:RoutingCommandFactory', action='testJstl')

        return map