import sys
import logging

class BaseWSGIApplication(object):
    def postInitialize(self,controller, kargs):
        pass

    def handle_exception(self, controller, e, isDebug=True):
        errorType = "%s"%(type(e).__name__)
        errorMessage = "%s"%(e)
        if not controller.request.get("callback"):
            controller.response.set_status(500,errorMessage)
            controller.response.headers['ErrorMessage'] = errorMessage
            controller.response.headers['ErrorType'] = errorType
            controller.handle_exception(e, isDebug)
        else:
            errorType = errorType.replace("'", "\"")
            errorMessage = errorMessage.replace("'", "\"")
            responseStr = controller.request.get("callback")+"({'errorType': '%s','errorMessage':'%s', 'textStatus': 500, 'errorTrace':''})"%(errorType, errorMessage)
            controller.response.out.write(responseStr)

    def handle_invalid_url_error(self, environ, request, response, mapperPath):
        raise TypeError('No routes match for %s.The method is %s. Provide a fallback to avoid this.'%(mapperPath, environ['REQUEST_METHOD']))

    def initializeCommandFactory(self, request, response, environ, session, mapperPath):
        # Match the path against registered routes.
        method = request.params.get('__method')
        if method:
            environ['REQUEST_METHOD'] = method
        mimetype = request.params.get('mimetype')
        if mimetype:
            request.accept = mimetype

        self.mapper._envset(environ)
        controller = None

        self.mapper._envset(environ)
        if request.path=='/favicon.ico':
            return
        kargs = self.mapper.match(mapperPath)
        if kargs is None:
            self.handle_invalid_url_error(environ, request, response, mapperPath)

        # Extract the module and controller names from the route.
        try:
            module_name, class_name = kargs['controller'].split(':', 1)
            del kargs['controller']
        except Exception ,e:
            logging.error(str(e))
            raise #TypeError('Controller is not set, or not formatted in the form "my.module.name:MyControllerName".')

        # Initialize matched controller from given module.
        try:
            __import__(module_name)
            module = sys.modules[module_name]
#            logging.info(str(module))
            controller = getattr(module, class_name)()
            controller.initialize(request, response)
            controller.setSession(session)
            #cors support in orangescape jquery api
            if request.headers.get('Origin'):
                response.headers['Access-Control-Allow-Origin'] = request.headers.get('Origin')
                response.headers['Access-Control-Allow-Methods'] = "GET, POST"
                response.headers['Access-Control-Allow-Headers'] = "*"
                response.headers['Access-Control-Allow-Credentials'] = "true"
                response.headers['Access-Control-Max-Age'] = "86400"
                
            if request.headers.get('_TIMEZONE'):
                from orangescape.model.server.ThreadLocal import TIMEZONE, setThreadData
                setThreadData(TIMEZONE, int(request.headers.get('_TIMEZONE')))

            self.postInitialize(controller, kargs)
        except Exception ,e:
            logging.error(str(e))
            raise #ImportError('Controller %s from module %s could not be initialized.' % (class_name, module_name))

        # Use the action set in the route, or the HTTP method.
        if 'action' in kargs:
            action = kargs['action']
            del kargs['action']
        else:
            action = environ['REQUEST_METHOD'].lower()
            if action not in ['get', 'post', 'head', 'options', 'put', 'delete', 'trace']:
                action = None

        return controller,action,kargs