import logging
import zlib
import uuid

from orangescape.model.engine.execution.ProcessEngine import ProcessNode
from orangescape.model.server.services.AppService import AppSetupService, copyChange

from orangescape.model.engine.exception.Error import ServerError
from orangescape.model.engine.exception.ErrorMessage import Server2001
import appconfig as config

class AppSetupProcessNode(ProcessNode):

    def enter(self):
        logging.debug('Creating application system models')
        self.sheet.getDataObject().ApplicationId = self.sheet.getSheetId()
        templateId = self.sheet.getValue('AppTemplate')
        releaseId = self.sheet.getValue('ReleaseTemplate')
        if releaseId:
            from orangescape.model.server.db.pickleUtil import unPickleStr
            releaseMetadata = self.session.getMetadata('Release')
            modelReleaseMetadata = self.session.getMetadata('ModelRelease')
            refCell = modelReleaseMetadata.getPublicCellMetadata('Release')

            releaseInstance = self.session.get(releaseId, releaseMetadata)
            releaseAppInst = self.session.get(releaseInstance.getValue('Application'), self.sheet.getSheetMetadata())
            logging.info('Creating application from Release Template %s(%s)'%(releaseInstance.getSheetName(), releaseInstance.getSheetId()))

            self.component.update(self.sheet, {'MetadataVersion':releaseAppInst.getValue('MetadataVersion')})
            fileName, pck_data = self.component.getAttachmentWithId(releaseInstance)
            data = unPickleStr(pck_data)

            service = AppSetupService(self.component, self.sheet)
            service.insertAppRecords(data)
            service.copySheet_n_Attachments(releaseInstance, refCell)

        elif templateId:
            from orangescape.model.server.util.privateserializer import SheetSerializer
            templateApp = self.session.get(templateId, self.sheet.getSheetMetadata())
            self.component.update(self.sheet, {'MetadataVersion':templateApp.getValue('MetadataVersion')})
            excludeListDict = {'Sheet008' : ['CellMetadata_99dcc7f8_f829_4da1_8d4f_a00c57684e45','Cell_001_002_006', 'Cell_029_002_006', 'Cell_015_002_006', 'Ced2a6420a28ae43539a38ced226abc6de', 'CellMetadata_c9e14075_5719_42b0_b62a_217477894f41', 'Cell_016_002_006', 'Cell_027_002_006']}
            if config.PLATFORM=='studio-kf':
                excludeListDict['SheetMetadata_ec3f229c_327b_443b_88d7_0ca275dc42a0'] = ['CellMetadata_d4d80b37_11d6_4088_bc05_14d7162aaebc']
            data = SheetSerializer(self.session, templateApp, excludeListDict, True).serializeApplication()
            service = AppSetupService(self.component, self.sheet)
            service.insertAppRecords(data)
            sheetAppRefCell = self.session.getMetadata('SheetMetadata').getPublicCellMetadata('Application')
            service.copySheet_n_Attachments(templateApp, sheetAppRefCell)
        else:
            self.component.update(self.sheet, {'MetadataVersion':config.metadataVersion})
            from orangescape.studio.sysmodels.primary.Application import application as d
            data = copyChange(d)
            service = AppSetupService(self.component, self.sheet)
            service.insertAppRecords(data)
            service.createSheet_n_Attachments()

        self.session.clearOne2ManyCache(self.sheet, self.session.getMetadata('SheetMetadata').getPublicCellMetadata('Application'))