"""A Perfomance Monitoring Module
Exported functions
    saveDebugData - To save JSON in gaeWSGI/servletWSGI
    startPerfMonitor - TO start the perfmonitor
    stopPerfMonitor - To Stop the perfmonitor
    startSessionFormulaMonitor - To start the perfmonitor for formula/session

Exported Class:
    PerfMonitor - A simple class for Complete Monitoring

"""


import logging
import time
try:
    from threading import local
    data = local()
except ImportError:
    data = {}
from orangescape.model.server.Context import getApplication
application =getApplication()
snapshotDef = None


def getActiveSnapShot(modelname,method,action=None):
    """
    To obtain the snapshot definition for the current request
    1. First use the thread local CurrentMonitor for the snapshot else updates it from  snapShotDict(Context.py)

    Returns the logFormula,logSession, cell metrics using snapshot_cellMetrics
    """
    global data
    global snapshotDef
    key = modelname+method+action
    logging.info(key)
    if hasattr(data.CurrentMonitor,"snapshot") and data.CurrentMonitor.snapshot is None:

        # from orangescape.model.server.snapshot import jsonapp
        # global snapshotDefList
        # snapshotDefList = jsonapp
        from orangescape.model.server.Context import getSnapshotAttachements
        snapshotDef = getSnapshotAttachements(data.CurrentMonitor.session,False)
        if snapshotDef is not None and snapshotDef.__contains__(key):
            snapshot = snapshotDef.get(key)
            if snapshot[key]['ModelName'] == modelname and snapshot[key]['ActionName'] == action and snapshot[key]['Method'] == method:
                data.CurrentMonitor.snapshot = snapshot
                data.CurrentMonitor.snapshotname = snapshot[key]['WatchPointName']
            else:
                data.CurrentMonitor.snapshot = None
                return (None, None, None)
        elif snapshotDef is not None and snapshotDef.__contains__(modelname+'allall'):
            key = modelname+'allall'
            snapshot = snapshotDef.get(key)
            if snapshot[key]['ModelName'] == modelname and snapshot[key]['ActionName'] == 'all' and snapshot[key]['Method'] == 'all':
                data.CurrentMonitor.snapshot = snapshot
                data.CurrentMonitor.snapshotname = snapshot[key]['WatchPointName']
                logging.info(snapshot[key]['WatchPointName'])
            else:
                data.CurrentMonitor.snapshot = None
                return (None, None, None)
        else:
            return (None, None, None)
    elif not hasattr(data.CurrentMonitor,"snapshot"):
        return (None, None, None)
    else:
        snapshot = data.CurrentMonitor.snapshot
    Cells_List = snapshot_cellMetrics(snapshot,snapshot.__getitem__('root') ,modelname, method, action)
    if Cells_List is not None:
        logSession = data.CurrentMonitor.snapshot[snapshot.__getitem__('root')].get('logSession')
        logFormula = data.CurrentMonitor.snapshot[snapshot.__getitem__('root')].get('logFormula')
        return (Cells_List,logSession,logSession)
    return (None, None, None)


def snapshot_cellMetrics(snapshot, key,modelname,method,action=None):
    """
    To obtain the cell metrics from the snapshot recursively
    """

    if snapshot.__contains__(key):
        if snapshot[key]['ModelName'] == modelname and (snapshot[key]['Method'] == method or snapshot[key]['Method'] == 'all')  and (snapshot[key]['ActionName'] == action or  snapshot[key]['ActionName'] == 'all'):
            Cells_List = []
            if snapshot[key].__contains__('WatchPointRef.Metrics') and snapshot[key].get('WatchPointRef.Metrics'):
                for key in  snapshot[key]['WatchPointRef.Metrics'].keys():
                    Cells_List.append(snapshot[key]['CellName'])
                return Cells_List
            else:
                global application
                sheet = application.getSheetMetadata(modelname)
                Cells_List = sheet.getPublicCellMetadataDict().keys()
                return Cells_List
        else:
            if snapshot[key].__contains__('WatchPoint_Ref.WatchPoint'):
                for j in snapshot[key]['WatchPoint_Ref.WatchPoint'].keys():
                    logging.info(j)
                    Cells_List = snapshot_cellMetrics(snapshot,j,modelname,method,action)
                    if Cells_List:
                        return Cells_List
                return None
            return None
    return None


def saveDebugData(session,DebugData,timeTaken,BaseWatchPoint,BaseMethod):

    """
    To save the constructed JSON as an attachment in the UserRequests Model
    """
    from orangescape.model.engine.Component import Component
    from orangescape.model.engine.execution.ExecutionTree import ExecutionTree
    global application
    UserRequestSheet = application.getSheetMetadata('UserRequests')
    userLoginModel = application.getSheetMetadata('UserLoginModel')
    referenceCell=UserRequestSheet.getPublicCellMetadata('UserLoginModelRef')
    component = Component(session)
    userInst = session.getUser()
    userloginId = userInst.getValue('UserLoginModel')
    userloginInst = session.get(userloginId , userLoginModel )
    instance = component.create(userloginInst ,referenceCell, {'UserId':userInst .getValue('SheetId'),'Company':userInst.getValue('Company'),'TimeTaken':timeTaken,'WatchPointName':BaseWatchPoint,'WatchPointMethod':BaseMethod})
    AttachmentId = str(BaseWatchPoint)+'.txt'
    component.updateAttachmentWithId(instance, AttachmentId, DebugData, tag=False)
    startAct = UserRequestSheet.getWorkFlowRoot().getActivityDefinitionList()[0]
    component.submit(instance,startAct)
    from appconfig import env
    if env == 'gae1':
        session.commit(True)
    else:
        session.commit()
    return True

def getCurrentMonitor():
    """
    To Fetch the current monitor
    """
    global data
    if hasattr(data, "CurrentMonitor") and data.CurrentMonitor:
        return data.CurrentMonitor
    else:
        return None


def startPerfMonitor(sheetname,method,actionName=None,sheetid=None,inputSheet=None,inputValues=None):
    """
    To verify, obtain the cell metrics and start the perfmonitor

    Returns
        PerfMonitor object or None
    """
    from appconfig import DEBUGGER_ENABLED
    if getCurrentMonitor() and DEBUGGER_ENABLED == 1:
        metrics,logSession,logFormula = getActiveSnapShot(sheetname,method,actionName)
        if metrics is not None:
            pm= PerfMonitor(sheetmetadataname=sheetname, method=method,actionName=actionName,metrics=metrics,sheetid=sheetid,logSession=logSession,logFormula=logFormula)
            pm.start(inputSheet=inputSheet,inputValues=inputValues)
            return pm
        else:
            return None
    else:
        return None


def stopPerfMonitor(pmObj,metrics=None):
    """
    To Stop the PerfMonitor
    """

    if pmObj is not None:
        pmObj.stop(metrics)

def startSessionFormulaMonitor(selfDict,formula=None):
    """
    To start the PerfMonitor for Formula as well as Session

    Returns
        PerfMonitor object or None
    """
    import appconfig as config
    if config.DEBUGGER_ENABLED == 1:
        currentMonitor = getCurrentMonitor()
        if currentMonitor and currentMonitor.logSession == 1.0 and selfDict['type'] == "Session":
            pm= PerfMonitor(sheetmetadataname=selfDict['sheetName'], method=selfDict['name'],logSession=1.0)
            pm.start(inputValues=selfDict['inputValues'])
            return pm
        elif currentMonitor and currentMonitor.logFormula == 1.0 and selfDict['name'] == "Formula":
            metrics,pm,actionName = None,None,None
            if formula is not None:

                if formula.getCellMetadata():
                    if formula.getCellMetadata().getCellName():
                        metrics = formula.getCellMetadata().getCellName()+' ( '+formula.getCellMetadata().getCellDisplayName()+' )'
                    else:
                        metrics = formula.getCellMetadata().getCellDisplayName()
                elif formula.getActivityDefinition() and (formula.getActivityDefinition().getProcessType() == 'Workflow' or formula.getActivityDefinition().getProcessType() == 'Command'):
                    metrics = selfDict['snapshot']+' ( '+formula.getActivityDefinition().getSheetName()+' in '+formula.getActivityDefinition().getProcessType()+' )'
                elif formula.getInputportMetadata() and formula.getInputportMetadata().getActivityDefinition().getProcessType() == 'Action':
                    from orangescape.model.server.util.metadatautil import getRootBranch
                    metrics =selfDict['snapshot'] +' ( '+formula.getInputportMetadata().getActivityDefinition().getNodeType()+' in '+getRootBranch(formula.getInputportMetadata().getActivityDefinition().getBranch()).getSheetName()+' )'
            if selfDict.__contains__('snapshot') and selfDict['snapshot'] is not None:
                actionName = selfDict['snapshot']
                metrics = selfDict['snapshot'] if metrics is None else metrics
            if metrics is not None:
                pm= PerfMonitor(sheetmetadataname=selfDict['sheetName'], method=selfDict['name'],actionName=actionName,metrics=metrics)
                pm.start(inputValues=selfDict['inputValues'])
                return pm
        else:
            return None
    else:
        return None

class PerfMonitor:
    """
    A Class for implementing the Perfmonance Monitoring

    Public Function
        start , stop, addChild, removeChild,toJSON,addJSON, timeTaken

    """
    parent = None
    children = None
    outputMetrics = None
    method = None
    snapshot = None
    sheetid = None
    sheetmetadataname = None
    startTime = 0
    endTime = 0
    payload = None
    actionName = None
    inputMetrics = None
    logSession = None
    session = None
    snapshotname = None
    metrics = None
    def __init__(self,sheetmetadataname =None,method=None,actionName=None,metrics=None,sheetid=None,logSession= None,logFormula=None,session=None):

        if method:
            self.method = method
        if actionName:
            self.actionName = actionName
        self.children = []
        self.sheetid = sheetid
        self.actionName = actionName
        self.sheetmetadataname = sheetmetadataname
        self.metrics = metrics
        self.logSession = logSession
        self.logFormula = logFormula
        self.session = session

    def addChild(self, child):
        self.children.append(child)

    def removeChild(self,child):
        if child in self.children:
            self.children.remove(child)

    def start(self,inputSheet=None,inputValues=None):
        self.startTime = time.time()
        global data
        if hasattr(data, "CurrentMonitor") and data.CurrentMonitor:
            self.parent = data.CurrentMonitor
            self.session = data.CurrentMonitor.session
            self.snapshotname = data.CurrentMonitor.snapshotname
            self.snapshot = data.CurrentMonitor.snapshot
            if self.parent.method == 'Main' and self.parent.actionName is None:
                self.parent.sheetmetadataname = self.sheetmetadataname
                self.parent.actionName = self.method
            self.parent.addChild(self)
            self.inputMetrics = dict()
            if inputValues is not None :
                    self.inputMetrics['InputValues'] = str(inputValues)
            logging.info(type(self.snapshot).__name__)
            if self.metrics is not None and type(self.metrics).__name__ == 'list':
                self.inputMetrics['inputSheet'] = dict()
                if inputSheet is not None:
                    for val in self.metrics:
                        self.inputMetrics['inputSheet'][str(val)] = str(inputSheet.getValue(val))
            data.CurrentMonitor = self
        if self.method == 'Main':

            data.CurrentMonitor = self
        logging.info(self.method)
        logging.info(self.snapshot)
    def stop(self,metrics=None):
        self.endTime = time.time()
        global data
        if self.parent:
            data.CurrentMonitor = self.parent
        self.outputMetrics = dict()
        if metrics is not None and self.metrics is not None and self.method in ['read','new','update','create','workflow','action','Get','Command']:
            self.sheetid = metrics.getSheetId()
            for val in self.metrics:
                self.outputMetrics[str(val)] = str(metrics.getValue(val))

        elif self.method == 'delete' and metrics is not None:
            for val in metrics:
                self.outputMetrics[str(val.getSheetId())] = str(val.getSheetMetadata().getSheetName())

        elif self.method == 'Formula'and metrics is not None:
            self.outputMetrics[str(self.metrics)] = str(metrics)

        elif self.method in ['GetList','Query','QueryDS','DBQueryDS','ReportDS','Report']:
            self.outputMetrics['No of Results Obtaned'] = str(metrics)
        elif self.method.startswith('serialize'):
            self.outputMetrics['argList'] = str(metrics)
        else:
            self.outputMetrics = None


    def timetaken(self):
        return  round(self.endTime - self.startTime,4)

    def addJSON(self, strBuffer):
        children = 0
        strBuffer.append("{'type':'%s','sheetid':'%s','sheetmetadataname':'%s','method':'%s','time':'%s'"%(self.method,self.sheetid,self.sheetmetadataname,self.actionName,self.timetaken()))
        if self.outputMetrics is not None:
            strBuffer.append(",'outputMetrics':%s"%(self.outputMetrics))
        if self.inputMetrics is not None:
            strBuffer.append(",'inputMetrics':%s"%(self.inputMetrics))
        for child in self.children:
            if children == 0:
                children = 1
                strBuffer.append(",'payload':[")
            else:
                strBuffer.append(",")
            child.addJSON(strBuffer)
            strBuffer.append("}")
        if children == 1:
            strBuffer.append("]")

    def toJSON(self, url=None, action=None, user=None, session=None):
        global data
        data.CurrentMonitor = None
        strBuffer = list()
        strBuffer.append("{'url':'" + (url if url else "") + "', ")
        strBuffer.append("'user':'" + (user if user else "") + "', ")
        strBuffer.append("'SessionId':'" + (session if session else "") + "', ")
        strBuffer.append("'PerfMonitor_parent':")
        self.addJSON(strBuffer)
        strBuffer.append("}}")
        return ''.join(strBuffer)

