import logging
import time
import os
import re

import webapp2
from google.appengine.ext import webapp
from google.appengine.api import memcache
from google.appengine.api import app_identity
from google.appengine.ext.webapp import template
from google.appengine.api import mail

from orangescape.model.server.UserAuthentication import UserAuthentication
from orangescape.model.server.Cookies import Cookies
from CommandFactory import CommandFactory
from orangescape.model.server.util.basicutil import gzip_decompress

from orangescape.model.engine.exception.Error import InstanceNotFoundError
import appconfig as config
from orangescape.model.server.util.ServerUtil import _get_class
from orangescape.model.server.util.metadatautil import searchModelByName

from CommandUtil import dispatchCommand, getUrlParams
from datetime import datetime, timedelta
import build
from orangescape.model.server.util.basicutil import gzip_compress,TZ
log = logging.getLogger(__name__)

class GAECommandFactory(webapp2.RequestHandler,CommandFactory):
#    def __init__(self):
#        from orangescape.model.server.GFSSession import PrimaryGFSSession as PrimarySession
#        self.setSession(PrimarySession(getApplication()))

    def logout(self,metadatatype,applicationid):
        cookies = Cookies(self)
        UserAuthentication(self.getSession()).logOutUser(self.userid)
        del cookies['sessionid']
        del cookies['userid']
        authInstance = _get_class(config.AUTHENTICATION)()
        self.redirect(authInstance.getLogoutUrl())

    def receiveMail(self, url=None, metadatatype=None, applicationid=None):
        import urllib
        message = mail.InboundEmailMessage(self.request.body)
        message_subject = message.subject
        message_body=""
        message_sender_list = re.split('<|>',message.sender)
        message_sender = message_sender_list[1] if len(message_sender_list) > 1 else message_sender_list[0]
        
        #separate the @ to get the model name
        requestPath = urllib.unquote(self.request.path)
        model_name=requestPath.split('/')[len(requestPath.split('/'))-1].split('@')[0]
        #separate - to get the content type
        modelNameList = model_name.split('-')
        model_name = modelNameList[0]
        
        contentType = 'text/plain'
        if len(modelNameList)>1 and modelNameList[1] == 'html':
            contentType = 'text/html'
            
        mail_bodies = message.bodies(contentType)
        for content_type, body in mail_bodies:
            message_body+=body.decode()
            
        if logging.getLogger().level == logging.DEBUG:
            logging.debug('mailing user: %s'%(message.sender))
            logging.debug('mailed content type: %s'%(contentType))
            logging.debug('Saving in model: %s'%(model_name))
       
        application =  self.getSession().getApplication()
        userinitialization=UserAuthentication(self.session, message_sender)
        userSurrogateId=userinitialization.authenticateUser()
        metadata = application.getSheetMetadata(model_name)
        instance = dispatchCommand(self.getSession(),'new', metadata, {'Summary':message_subject,'Description':message_body})
        
        if hasattr(message, 'attachments'):
            childModelName = config.MAIL_ATTACHMENT_MODEL
            if application.getSheetMetadataDict().__contains__(childModelName):
                childMetadata = application.getSheetMetadata(childModelName)
                attachments = message.attachments
                if childMetadata.getPublicCellMetadataDict().__contains__(model_name):
                    refCell = childMetadata.getPublicCellMetadata(model_name)
                    for items in attachments:
                        attachmentName = items[0]
                        attachmentContent = items[1]
                        payload = attachmentContent.payload.decode(attachmentContent.encoding)
                        childInstance = dispatchCommand(self.getSession(),'create', instance, refCell, {})
                        dispatchCommand(self.getSession(),'updateAttachmentWithId', childInstance, attachmentName, payload)
                else:
                    logging.warn("The mail attachment model '%s' doesn't contain the reference cell named '%s'"%(childModelName, model_name))
            else:
                logging.warn("The mail contains attachments but the model '%s' is missing the current application"% childModelName)
                
        """Since its newly getting created submit from Start activity"""
        rootBranch = metadata.getWorkFlowRoot()
        if rootBranch:
            startActivity = rootBranch.getActivityDefinitionList()[0]
            dispatchCommand(self.getSession(),'submit', instance, startActivity)
        self.__commit__()

    def __getSubscriptionList__(self, application):
        metadataContext = self.getSession().getApplication()
        appSubscription = metadataContext.getSheetMetadata('ApplicationSubscription')
        refCell = appSubscription.getPublicCellMetadata('Application')
        return [appSub for appSub in self.getSession().getList(application, refCell) if appSub.getValue('Status') == 'ActivityDefinition_6a46e5d5_5951_4c25_9539_a3deaf9eeb28']

    def openModel(self):
        query_string=self.request.query_string
        params = self.getQueryStringParams(query_string)
        metadataContext = self.getSession().getApplication()

        appSubscriptionId = params.get('applicationsubscriptionid')
        if not appSubscriptionId:
            msg = 'Studio can\'t be loaded without applicationsubscriptionid'
            self.response.out.write(msg)
            return msg

        releaseId = params.get('releaseid')
        needsMigration = 0
        if params.get('migrate'):
            needsMigration = int(params.get('migrate'))
        initParams = {'ApplicationSubscriptionId':appSubscriptionId, 'url_scheme':self.request.environ.get('wsgi.url_scheme')}
        versionNumber = "trunk"

        appSubscriptionMetadata = metadataContext.getSheetMetadata('ApplicationSubscription')
        applicationMetadata = metadataContext.getSheetMetadata('Application')
        currentSubscription = self.getSession().get(appSubscriptionId, appSubscriptionMetadata)
        applicationId = currentSubscription.getValue('Application')
        application = self.getSession().get(applicationId, applicationMetadata)

        """Blocking ESIC apps to load the studio using trial. Needs to be removed once all the ESIC apps migrated to NEW Studio."""
#        if applicationId in ('Application_3e33f614_dc1c_11de_b6ab_8d4e356e409e','Application_6a7f0985_c2c0_11de_af18_39d4b6a23447','Application_9d997480_b2f8_11de_9786_d7bab8d51d7c','Application_c66ef1f8_b6ee_11de_96dd_65dc25a820a3','Application_e6f5ca82_0a7a_11df_9716_af2651d75c85','Application_e9a76b33_da55_11de_b46b_83553b597d05'):
#            oldAppspotUrl = 'http://recent.latest.os-dev.appspot.com/'
#            self.response.out.write('Your application requires an older version of Studio. <br>Launch your application Using <a href="%s">Previous Version</a>'%(oldAppspotUrl))
#            return 'Your application requires an older version of Studio. <br>Launch your application Using <a href="%s">Previous Version</a>'%(oldAppspotUrl)
        """********************************************************************************"""
        if (not(currentSubscription.getValue('Tenant') == self.getSession().getUser().getValue('Company') and currentSubscription.getValue('AssignedTo') == self.getSession().getUser().getValue('UserId').split('@')[0]) and  needsMigration != 1 and self.getSession().getUser().getValue('SheetId') != application.getValue('Owner')) or  currentSubscription.getValue('Status')!='ActivityDefinition_6a46e5d5_5951_4c25_9539_a3deaf9eeb28':
            msg = 'Your Application %s needs to be migrated, kindly contact your application owner %s'%(application.getSheetName(), application.getValue('ApplicationOwner'))
            self.response.out.write(msg)
            return msg
        elif (not(currentSubscription.getValue('Tenant') == self.getSession().getUser().getValue('Company') and currentSubscription.getValue('AssignedTo') == self.getSession().getUser().getValue('UserId').split('@')[0]) and  needsMigration != 1 and self.getSession().getUser().getValue('SheetId') == application.getValue('Owner')) or  currentSubscription.getValue('Status')!='ActivityDefinition_6a46e5d5_5951_4c25_9539_a3deaf9eeb28':
            msg = 'Your Application %s needs to be migrated, kindly use Migrate and Launch button to migrate.'%(application.getSheetName())
            self.response.out.write(msg)
            return msg
        elif ((not(currentSubscription.getValue('Tenant') == self.getSession().getUser().getValue('Company') and currentSubscription.getValue('AssignedTo') == self.getSession().getUser().getValue('UserId').split('@')[0])) and  needsMigration == 1 and self.getSession().getUser().getValue('SheetId') == application.getValue('Owner')) or  currentSubscription.getValue('Status')!='ActivityDefinition_6a46e5d5_5951_4c25_9539_a3deaf9eeb28':
            from orangescape.model.engine.Component import Component
            from orangescape.model.server.Repair import migrateApplication
            import appconfig
            component = Component(self.session)
            initParams['NeedsMigration'] = 1
            migrateApplication(component,applicationId)
            component.update(application, {'MetadataVersion':appconfig.metadataVersion})
            self.session.commit()
        elif self.getSession().getUser().getValue('SheetId') == application.getValue('Owner') and needsMigration == 1:
            import appconfig
            from orangescape.model.engine.Component import Component
            component = Component(self.session)
            initParams['NeedsMigration'] = 1
            component.update(application, {'MetadataVersion':appconfig.metadataVersion})
            self.session.commit()
        subScipList = self.__getSubscriptionList__(application)
        if len(subScipList)==1:
            initParams['IsSingleUser'] = 1
        if currentSubscription.getValue('ApplicationOwner') == self.getSession().getUser().getValue('SheetId'):
            initParams['AppOwner'] = 1

        if releaseId:
            releaseMetadata = metadataContext.getSheetMetadata('Release')
            parentSheet = self.getSession().get(releaseId, releaseMetadata)
            versionNumber = parentSheet.getValue('VersionNumber')
            referenceCell = metadataContext.getSheetMetadata('ModelRelease').getPublicCellMetadata('Release')
        else:
            releaseId = ''
            parentSheet = application
            referenceCell = metadataContext.getSheetMetadata('SheetMetadata').getPublicCellMetadata('Application')

        modelNames, sheetListStr = self.__getSheetsList__(parentSheet, referenceCell)
        version_appspot_url = ""
        if build.VersionAppspotUrl.get(versionNumber):
            version_appspot_url = build.VersionAppspotUrl.get(versionNumber)
        templatePath = os.path.dirname(__file__).split('model')[0]
        templatePath = os.path.join(templatePath,'studio.html')
        jsonstr = template.render(templatePath, {'applicationid':applicationId, 'sheetidlist':sheetListStr, 'userid':self.userid, 'sheetid':'', 'releaseid':releaseId, 'initParams':initParams, 'version_appspot_url':version_appspot_url})
        self.response.headers['Cache-Control'] = 'no-cache'
        self.response.out.write(jsonstr)

    def reloadModel(self):
        query_string=self.request.query_string

        params = self.getQueryStringParams(query_string)
        metadataContext = self.getSession().getApplication()

        workingSheetId = params.get('workingsheetid')
        applicationId = params.get('applicationid')
        sheetId = params.get('sheetid')
        releaseId = params.get('releaseid')
        modelReleaseId = params.get('modelreleaseid')
        forceUnlock = params.get('forceUnlock')
        lastModifiedAt = float(params.get('lastsyncat'))
        selectedSheetId = None
        sheet = None
        if releaseId:
            parentMetadata = metadataContext.getSheetMetadata('Release')
            metadata = metadataContext.getSheetMetadata('ModelRelease')
            referenceCell = metadata.getPublicCellMetadata('Release')

            parentSheet = self.getSession().get(releaseId, parentMetadata)
            applicationId = parentSheet.getValue('Application')
            if modelReleaseId:
                sheet = self.getSession().get(modelReleaseId, metadata)
                selectedSheetId = sheet.getValue('Attachment')[:-4]
        else:
            parentMetadata = metadataContext.getSheetMetadata('Application')
            metadata = metadataContext.getSheetMetadata('SheetMetadata')
            referenceCell = metadata.getPublicCellMetadata('Application')

            parentSheet = self.getSession().get(applicationId, parentMetadata)
            if sheetId:
                sheet = self.getSession().get(sheetId, metadata)
                selectedSheetId = sheetId
            releaseId = ''
        try:
            if sheet:
                if forceUnlock is None and not (sheet.getValue('ModifiedBy') is None or sheet.getValue('ModifiedBy') == self.userid):
                    lockedBy = self.getSession().get(sheet.getValue('ModifiedBy'), metadataContext.getSheetMetadata('User'))
                    self.response.headers['Cache-Control'] = 'no-cache'
                    self.response.out.write("{'locked': 'Model %s is locked by %s'}"%(sheet.getSheetName(), lockedBy.getSheetName()))
                    return
                if sheet.getValue('ModifiedBy') is None or forceUnlock is not None:
                    logging.info('Locking the sheet %s '%(sheet.getSheetId()))
                    dispatchCommand(self.getSession(), 'update', sheet, {'ModifiedAt':sheet.getValue('ModifiedAt')})

            if workingSheetId and selectedSheetId != workingSheetId:
                workingSheet = self.getSession().get(workingSheetId, metadata)
                logging.info('Releasing the sheet %s '%(workingSheet.getSheetId()))
                dispatchCommand(self.getSession(), 'update', workingSheet, {'ModifiedBy':None, 'ModifiedAt':workingSheet.getValue('ModifiedAt')})

            includes = params.get('includes')
            if includes:
                includes = eval(includes)
            modelNames, sheetListStr = self.__getSheetsList__(parentSheet, referenceCell, lastModifiedAt, includes)
            logging.debug('Reload model response %s'%(sheetListStr))
            self.getSession().commit()
            self.response.headers['Cache-Control'] = 'no-cache'
            self.response.out.write(sheetListStr)
        except Exception, e:
            raise

    def __getSheetsList__(self, parent, refCell, lastModifiedAt=None, includes=None, excludes=None):
        from cStringIO import StringIO

        allSheetList = self.getSession().getList(parent, refCell)
        if lastModifiedAt:
            sheetList = [sheet for sheet in allSheetList if sheet.getValue('ModifiedAt') > lastModifiedAt or (includes and includes.__contains__(sheet.getSheetId()))]
            logging.debug('Last modified details %s'%([(sheet.getValue('Attachment')[:-4], sheet.getValue('ModifiedAt')) for sheet in allSheetList if (includes and includes.__contains__(sheet.getValue('Attachment')[:-4]))]))
        else:
            sheetList = allSheetList

        buff = StringIO()
        buff.write("{'lockinfo' :{")
        for sheet in allSheetList:
            if sheet.getValue('ModifiedBy'):
                buff.write("'%s':'%s', "%(sheet.getValue('Attachment')[:-4], sheet.getValue('ModifiedBy')))
            else:
                buff.write("'%s':None, "%(sheet.getValue('Attachment')[:-4]))

        buff.write("},'sheetinfo' :{")
        modelNames = {}
        for sheet in sheetList:
            if (not excludes or not excludes.__contains__(sheet.getSheetId())):
                self.__write_sheet_info__(parent, refCell, sheet, buff)
                modelNames[sheet.getSheetName().upper()] = sheet.getValue('Attachment')[:-4]
        buff.write("}}")
        sheetListStr = buff.getvalue()
        return modelNames, sheetListStr

    def __write_sheet_info__(self, parent, refCell, sheet, buff):
        id = sheet.getValue('Attachment')[:-4]
        buff.write("'%s':{"%(id))
        from cStringIO import StringIO
        details = StringIO()

        details.write("'ModifiedAt':%.7f, "%(sheet.getValue('ModifiedAt')+0.0000001))
        if parent.getSheetMetadata().getSheetName()=='Release':
            details.write("'ModelReleaseId':'%s', "%(sheet.getSheetId()))
        buff.write('%s}, '%(details.getvalue()))

    def clearCache(self):
        logging.info('Clearing memcache')
        query_string=self.request.query_string
        queryStringDict = self.getQueryStringParams(query_string)
        cachekey = queryStringDict.get('cachekey')
        if cachekey:
            memcache.delete(cachekey)

    def checkout(self, metadatatype, applicationid, parentsheetmetadataid,taskid,sheetmetadataid,url):
        expirationTime = time.ctime(time.time()+(60*60*24*365))
        try:
            url=getUrlParams(self.request.path)
            url.pop()
            #Application001/1/54321.123/Sheet008/ActivityDefinition_2/Sheet008/Application_2/Cell_001_002_006/Sheet001/SheetMetadata_154.pri/checkout
            metadataType=url.pop(0)
            modifiedIn = url[2]

            fileName=url.pop()
            sheetMetadataId=url.pop()
            referenceCellId=url.pop()
            parentSheetId = url.pop()
            parentSheetMetadataId=url.pop()
            authorization = None
            if 'Authorization' in self.request.headers:
                    from CommandUtil import authenticateIntegration
                    authorization= self.request.headers["Authorization"]
                    logging.info(authorization)
                    responseDict=authenticateIntegration(self.getSession(),authorization)
                    logging.info(responseDict)
            else:
                from orangescape.model.server.util.AuthUtil import authenticate
                session = self.getSession()
                userid = authenticate(self)
                if userid:
                    isAuthenticated = True
                    session.setUser(userid)

            applicationObj = self.getSession().getApplication()
            self.getSession().setModifiedIn(modifiedIn)

            sheetMetadata=applicationObj.getSheetMetadataById(sheetMetadataId)
            parentSheetMetadata=applicationObj.getSheetMetadataById(parentSheetMetadataId)
            parentSheet=self.getSession().get(parentSheetId, parentSheetMetadata)
            referenceCell = sheetMetadata.getCellMetadataById(referenceCellId)

            fileContent = dispatchCommand(self.getSession(),'getAttachment', parentSheet, referenceCell, fileName)

            self.response.headers['Content-Type'] = 'text/plain'
            self.response.headers['Cache-Control'] = 'public'
            self.response.headers['Expires'] = expirationTime
            self.response.headers['Content-Disposition'] = 'attachment; filename="%s"' % fileName
            self.response.out.write(fileContent)
        except InstanceNotFoundError, e:
#                self.response.set_status(404, 'Instance not found %s'%(e))
            self.response.headers['Content-Type'] = 'text/plain'
            self.response.headers['Expires'] = expirationTime
            self.response.headers['Content-Disposition'] = 'attachment; filename="%s"' % fileName
            import pickle
            self.response.out.write(pickle.dumps({}, 0))
        except Exception, e:
            logging.error('Error during checkout %s'%(e))
            self.response.set_status(500)

    def commit(self, applicationid, metadatatype, parentsheetmetadataid, taskid, sheetmetadataid, url):
        content = self.request.body
        fileContent = gzip_decompress(content)
        query_string = self.request.query_string
        url=getUrlParams(self.request.path)
        url.pop()
        metadataType=url.pop(0)
        modifiedIn = url[2]
        params = self.getQueryStringParams(query_string)
        applicationObj = self.getSession().getApplication()
        self.getSession().setModifiedIn(modifiedIn)
        forceUnlock = params.get('forceUnlock')

        fileName = url.pop()
        sheetMetadataId = url.pop()
        referenceCellId = url.pop()
        parentSheetId = url.pop()
        parentMetadataId = url.pop()

        sheetMetadata = applicationObj.getSheetMetadataById(sheetMetadataId)
        parentMetadata = applicationObj.getSheetMetadataById(parentMetadataId)
        referenceCell = sheetMetadata.getCellMetadataById(referenceCellId)
        parentSheet = self.getSession().get(parentSheetId, parentMetadata)

        isNewModel = False
        if fileName.endswith('.pri'):
            sheetParams = {}
            for k, v in params.iteritems():
                if k != 'ModifiedAt' and k!= 'ModifiedBy' and sheetMetadata.getPublicCellMetadataDict().has_key(k):
                    sheetParams[k] = v

            sheetId = fileName[:-4]
            sheet = dispatchCommand(self.getSession(), 'getAttachmentInstance', parentSheet, referenceCell, fileName)
            lastModifiedAt = params.get('ModifiedAt')
            lastSyncAt = params.get('lastsyncat')
#            logging.debug('LastModifiedAt %s lastSyncAt %s'%(lastModifiedAt, lastSyncAt))
            if forceUnlock is None and sheet and sheet.getValue('ModifiedBy') and sheet.getValue('ModifiedBy') != self.userid:
                lockedUser = self.session.get(sheet.getValue('ModifiedBy'), applicationObj.getSheetMetadata('User'))
                errMsg = "{'Error':%s }"%(['Save failed! This model is Locked by the user %s. He might unlock the model and loaded '%(lockedUser.getSheetName())])
                self.response.out.write(errMsg)
                return

            if lastModifiedAt and sheet:
                logging.debug('Save %s %s %s'%(float(lastModifiedAt) > sheet.getValue('ModifiedAt'), (lastModifiedAt), sheet.getValue('ModifiedAt')))
            if forceUnlock is None and sheet and lastModifiedAt and float(lastModifiedAt) < sheet.getValue('ModifiedAt'):
                errMsg = "{'Error':%s}"%(['Save failed! You have older version of the model. Which might be loaded from the cache data.', 'Please clear the cache and reload the model from \"Server\" and Save the changes.'])
                self.response.out.write(errMsg)
                return

            if not sheet:
                isNewModel = True
                newSheetName = sheetParams.get('SheetName')
                sheetNames = [existingSheet.getSheetName().upper() for existingSheet in self.getSession().getList(parentSheet, referenceCell)]
                if sheetNames.__contains__(newSheetName.upper()):
                    return 'Model with name "%s" already exists. Please Rename & save.'%(newSheetName)

#                logging.debug('Creating new sheet with the id %s'%(sheetId))
                if sheetMetadataId=='Sheet001':
                    sheet = dispatchCommand(self.getSession(),'create', parentSheet, referenceCell, sheetParams, sheetId)
                else:
                    sheet = dispatchCommand(self.getSession(),'create', parentSheet, referenceCell, sheetParams)
#                logging.debug('New Sheet created %s'%(sheet.getSheetName()))
            elif len(sheetParams)>0:
                """Updating sheet info"""
#                logging.debug('Updating sheet info %s'%(sheetParams))
                sheet = dispatchCommand(self.getSession(), 'update', sheet, sheetParams)
            else:
                sheet = dispatchCommand(self.getSession(), 'update', sheet, {})
            dispatchCommand(self.getSession(),'updateAttachmentWithId', sheet, fileName, fileContent, params.get('tag'))
        else:
#            logging.debug('Updating attachment %s %s %s'%(parentSheet.getSheetName(), referenceCell, fileName))
            dispatchCommand(self.getSession(),'updateAttachment', parentSheet, referenceCell, fileName, fileContent, params.get('tag'))

        if params.get('release'):
            logging.debug('Releasing sheet %s'%(sheet.getSheetName()))
            dispatchCommand(self.getSession(),'update', sheet, {'ModifiedBy':None, 'ModifiedAt':sheet.getValue('ModifiedAt')})

        responseStr = ['{']
        callback = params.get('slcallback')
        if callback:
            responseStr.append("'slcallback':'%s', "%(callback))
        if fileName.endswith('.pri'):
            selectedSheetId = params.get('selectedsheetid')
            if selectedSheetId:
                selectedSheet = self.getSession().get(selectedSheetId, sheetMetadata)
                if forceUnlock is not None or selectedSheet.getValue('ModifiedBy') is None or selectedSheet.getValue('ModifiedBy') == self.userid:
                    modelNames, sheetListStr = self.__getSheetsList__(parentSheet, referenceCell, lastSyncAt, [selectedSheetId], [sheet.getSheetId()])
                    responseStr.append("'syncresponse' : {'ModifiedAt':%.7f}, 'reloadresponse' : %s"%(sheet.getValue('ModifiedAt')+0.0000001, sheetListStr))
                    if (forceUnlock is not None and selectedSheet.getValue('ModifiedBy') != self.userid) or selectedSheet.getValue('ModifiedBy') is None:
                        logging.debug('Locking the sheet %s '%(selectedSheet.getSheetId()))
                        dispatchCommand(self.getSession(), 'update', selectedSheet, {'ModifiedAt':selectedSheet.getValue('ModifiedAt')})
                    dispatchCommand(self.getSession(), 'update', sheet, {'ModifiedBy':None, 'ModifiedAt':sheet.getValue('ModifiedAt')})
                else:
                    lockedBy = self.getSession().get(selectedSheet.getValue('ModifiedBy'), applicationObj.getSheetMetadata('User'))
                    responseStr.append("'syncresponse' : {'ModifiedAt':%.7f}, 'reloadresponse' : {'locked': 'Model %s is locked by %s'}"%(sheet.getValue('ModifiedAt')+0.0000001, selectedSheet.getSheetName(), lockedBy.getSheetName()))
            else:
#                responseStr = "{'syncresponse' : {'ModifiedAt':%.7f}}"%(sheet.getValue('ModifiedAt')+0.0000001)
                responseStr.append("'syncresponse' : {'ModifiedAt':%.7f}"%(sheet.getValue('ModifiedAt')+0.0000001))
            if sheetMetadataId != 'Sheet001' and isNewModel:
                """New model created under release so pass the ModelReleaseId to the client(SilverLight)"""
                responseStr.append(",'ModelReleaseId' : '%s'"%(sheet.getSheetId()))
        self.getSession().commit()
        responseStr.append('}')
        self.response.out.write(''.join(responseStr))

    def serializeApplication(self,metadatatype,applicationid,parentsheetmetadataid,taskid,sheetmetadataid,url):
        """GET /Instances/id: Show a specific item."""
        url=getUrlParams(self.request.path)
        query_string = self.request.query_string
        authorization = None
        if 'Authorization' in self.request.headers:
                from CommandUtil import authenticateIntegration
                authorization= self.request.headers["Authorization"]
                logging.info(authorization)
                responseDict=authenticateIntegration(self.getSession(),authorization)
                logging.info(responseDict)
        else:
            from orangescape.model.server.util.AuthUtil import authenticate
            session = self.getSession()
            userid = authenticate(self)
            if userid:
                isAuthenticated = True
                session.setUser(userid)

        params = {}
        if query_string:
            params = self.getQueryStringParams(query_string)
            if params.has_key('excludeList'):
                params = eval(params['excludeList'])
        else:
            params = {'Sheet008' : ['CellMetadata_99dcc7f8_f829_4da1_8d4f_a00c57684e45', 'Cell_001_002_006', 'Cell_016_002_006']}
        #Application001/1/Sheet008/ActivityDefinition_2/Sheet008/Application_2/application
        url.pop()
        url.pop(0)
        modifiedIn = url[2]

        sheetId=url.pop()
        sheetMetadataId=url.pop()

        applicationObj = self.getSession().getApplication()
        self.getSession().setModifiedIn(modifiedIn)
        sheetMetadata=applicationObj.getSheetMetadataById(sheetMetadataId)
        sheet=self.getSession().get(sheetId, sheetMetadata)
        from orangescape.model.server.util.privateserializer import SheetSerializer

        appData = SheetSerializer(self.getSession(), sheet, params, True).serializeSubscription()
        import cPickle as pickle
        appPck = pickle.dumps(appData, 0)
        self.response.out.write(appPck)

    def serializeFormData(self, metadatatype, applicationid, parentsheetmetadataid, taskid, sheetmetadataid, url):
        """GET /Instances/id: Show a specific item."""
        url=getUrlParams(self.request.path)
        url.pop()
        url.pop(0)
        modifiedIn = url[2]

        sheetId=url.pop()
        sheetMetadataId=url.pop()

        applicationObj = self.getSession().getApplication()
        self.getSession().setModifiedIn(modifiedIn)
        sheetMetadata=applicationObj.getSheetMetadataById(sheetMetadataId)
        sheet=self.getSession().getIfExists(sheetId, sheetMetadata)
        formData = {}
        if sheet:
            from orangescape.model.server.util.privateserializer import SheetSerializer

            formMetadata = self.getSession().getApplication().getSheetMetadata('Form')
            excludeList  = {'Sheet001' : [refCell.getSheetId() for refCell in sheetMetadata.getOneToManyList() if refCell.getSheetMetadata().getSheetName()!='Form'],
                            formMetadata.getSheetId() : [refCell.getSheetId() for refCell in formMetadata.getOneToManyList() if refCell.getSheetMetadata().getSheetName()!='Form']
                            }
            formData = SheetSerializer(self.getSession(), sheet, excludeList, True).serializeSheet()
        import cPickle as pickle
        formPck = pickle.dumps(formData, 0)
        self.response.out.write(formPck)

    def modelversions(self, applicationid, metadatatype, parentsheetmetadataid, taskid, sheetmetadataid, url):
        urlParamList=getUrlParams(self.request.path)
        query_string = self.request.query_string
        url=getUrlParams(self.request.path)
        url.pop()
        downloadUrl = '/'.join(url)
        url.pop(0)

        sheetId=url.pop()
        sheetMetadataId=url.pop()
        applicationObj = self.getSession().getApplication()

        sheetMetadata=applicationObj.getSheetMetadataById(sheetMetadataId)
        sheet=self.getSession().get(sheetId, sheetMetadata)

        baseQuery = {}
        params = self.getQueryStringParams(query_string)
        orderBy = params.get('orderby') or 'versionedAt'
        limit = params.get('limit')
        limit=int(limit) if limit else 50

        isDescending = params.get('descending')
        if isDescending:
            isDescending = eval(isDescending)
        else:
            isDescending = True

        versionList = self.getSession().getAttachmentVersionList(sheetId, {}, orderBy=orderBy, isDescending = isDescending, limit=limit)
        template_values = {
            'modelName': sheet.getSheetName(),
            'versionList': versionList,
            'downloadurl':downloadUrl+'/downloadversion',
            'replaceurl':downloadUrl+'/replaceattachment',
            'replaceallurl':downloadUrl+'/replaceallattachment'
            }
        templatePath = os.path.dirname(__file__).split('model')[0]
        templatePath = os.path.join(templatePath,'templates','viewversions.html')
        response  = template.render(templatePath, template_values)
        self.response.out.write(response)

    def downloadversion(self, applicationid, metadatatype, parentsheetmetadataid, taskid, sheetmetadataid, url):
        urlParamList=getUrlParams(self.request.path)
        query_string = self.request.query_string
        url=getUrlParams(self.request.path)
        url.pop()
        url.pop(0)

        sheetId=url.pop()
        sheetMetadataId=url.pop()
        applicationObj = self.getSession().getApplication()

        params = self.getQueryStringParams(query_string)
        versionId = params.get('versionid')
        version = self.getSession().getVersion(versionId)
        fileName = sheetId+'.pri',
        response = gzip_compress(version)
        self.response.headers['Content-Disposition'] = 'attachment; filename="%s"' % fileName
        self.response.out.write(response)

    def replaceAllAttachment(self,applicationid, metadatatype, parentsheetmetadataid, taskid, sheetmetadataid, url):
        url=getUrlParams(self.request.path)
        query_string = self.request.query_string
        url.pop()
        url.pop(0)

        sheetId=url.pop()
        sheetMetadataId=url.pop()
        url.pop()
        appId= url.pop()
        applicationObj = self.getSession().getApplication()
        params = self.getQueryStringParams(query_string)
        sheetMetadata=applicationObj.getSheetMetadataById(sheetMetadataId)
        sheet=self.getSession().get(sheetId, sheetMetadata)
        appobj=self.getSession().get(appId, applicationObj.getSheetMetadataById(url.pop()))

        if sheet.getValue('ModifiedBy') is None:
            dispatchCommand(self.getSession(), 'update', sheet, {'ModifiedBy':None})
            self.getSession().commit()
        elif sheet.getValue('ModifiedBy') != self.userid:
            lockedBy = self.getSession().get(sheet.getValue('ModifiedBy'), applicationObj.getSheetMetadata('User'))
            dispatchCommand(self.getSession(), 'update', sheet, {'ModifiedBy':None})
            self.getSession().commit()
            logging.error('Model %s is Unlocked which is locked by %s'%(sheet.getSheetName(), lockedBy.getSheetName()))
            self.response.out.write('Model %s is Unlocked which is locked by %s \n'%(sheet.getSheetName(), lockedBy.getSheetName()))
        else:
            dispatchCommand(self.getSession(), 'update', sheet, {})
            self.getSession().commit()

        applicationSheetRefCell = sheetMetadata.getPublicCellMetadata('Application')
        modelList = self.getSession().getList(appobj, applicationSheetRefCell)
        versionmodel=self.getSession().getVersionFromDS( params.get('versionid'))
        for model  in modelList:
            if sheetId != model.getSheetId():
                modelId=model.getSheetId()
                baseQuery = {}
                params = self.getQueryStringParams(query_string)
                orderBy = 'versionedAt'
                limit= 1
                conditiondict = {}
                keylist=('versionedAt', '<')
                conditiondict[keylist] =  versionmodel.versionedAt
                isDescending = True
                versionList = self.getSession().getAttachmentVersionList(modelId, conditiondict, orderBy=orderBy, isDescending = isDescending, limit=limit)
                if len(versionList)>0:
                    for version in versionList:
                        versionId = version.key()
                        self.getSession().replaceAttachment(modelId, versionId)
                        self.response.out.write('Version Replaced for model %s at %s \n'%(model.getValue('SheetName'),version.versionedAt))
                else:
                    self.response.out.write('Version not avialable for model %s \n'%(model.getValue('SheetName')))
            else:
                versionId = params.get('versionid')
                self.getSession().replaceAttachment(sheetId, versionId)
                self.response.out.write('Version Replaced for model %s \n'%(sheet.getValue('SheetName')))

    def replaceAttachment(self, applicationid, metadatatype, parentsheetmetadataid, taskid, sheetmetadataid, url):
        url=getUrlParams(self.request.path)
        query_string = self.request.query_string
        url.pop()
        url.pop(0)

        sheetId=url.pop()
        sheetMetadataId=url.pop()
        applicationObj = self.getSession().getApplication()

        sheetMetadata=applicationObj.getSheetMetadataById(sheetMetadataId)
        sheet=self.getSession().get(sheetId, sheetMetadata)

        if sheet.getValue('ModifiedBy') is None:
            dispatchCommand(self.getSession(), 'update', sheet, {'ModifiedBy':None})
            self.getSession().commit()
        elif sheet.getValue('ModifiedBy') != self.userid:
            lockedBy = self.getSession().get(sheet.getValue('ModifiedBy'), applicationObj.getSheetMetadata('User'))
            logging.error('Could not replace version. Model %s is locked by %s'%(sheet.getSheetName(), lockedBy.getSheetName()))
            self.response.out.write('Could not replace version. Model %s is locked by %s'%(sheet.getSheetName(), lockedBy.getSheetName()))
        else:
            dispatchCommand(self.getSession(), 'update', sheet, {})
            self.getSession().commit()


        params = self.getQueryStringParams(query_string)
        versionId = params.get('versionid')
        self.getSession().replaceAttachment(sheetId, versionId)
        self.response.out.write('Version Replaced')

    def themeUpload(self):
        import httplib
        host = "deploy.orangescape.com"
        h = httplib.HTTPConnection(host)
        themeFile = self.request.POST['themefile']
        themeFileName = themeFile.filename
        themeContent = themeFile.value
        userId = self.request.POST['userId']
        themeId = self.request.POST['themeId']

        BOUNDARY = '--THE_FORM_BOUNDARY--'
        content_type = 'multipart/form-data; boundary=%s' % BOUNDARY

        selector = "/theme?userId=%s&themeId=%s"%(userId, themeId)
        h.request('POST', selector, themeContent,{'content-type':content_type})
        res = h.getresponse()
        #update the theme status
        applicationObj = self.getSession().getApplication()
        theme = self.getSession().get(themeId, applicationObj.getSheetMetadata("Themes"))
        if res.status!=200:
            dispatchCommand(self.getSession(), 'update', theme, {"Attachment": "Error", "SheetName": themeFileName})
        else:
            dispatchCommand(self.getSession(), 'update', theme, {"Attachment": "Success", "SheetName": themeFileName})
        self.getSession().commit()   
        self.redirect(self.request.referer)


    def userSync(self, metadatatype, applicationid):
        import orangescape.custom.usersync.config as userconfig
        if userconfig.OAUTH_CONSUMER_KEY is not None and userconfig.OAUTH_CONSUMER_SECRET is not None:
            from orangescape.custom.usersync.syncuser import usersync
            applicationObj = self.getSession().getApplication()
            sheetDict=applicationObj.getSheetMetadataDict()
            userSyncDict = dict()
            userSyncDict['USER_SHEET_METADATA_ID'] = sheetDict.get('User')
            userSyncDict['COMPANY_META_DATA_ID'] = sheetDict.get('Company')
            if sheetDict.__contains__('Group'):
                userSyncDict['GROUP_SHEET_METADATA_ID'] = sheetDict.get('Group')
                userSyncDict['GROUP_STATUS_VALUE']=applicationObj.getSheetMetadataById(sheetDict.get('Group')).getStartActivity().getSheetId()
            if sheetDict.__contains__('GroupUserSubscription'):
                userSyncDict['GROUP_USER_SHEET_METADATA_ID'] = sheetDict.get('GroupUserSubscription')
                userSyncDict['GROUP_USER_STATUS_VALUE']=applicationObj.getSheetMetadataById(sheetDict.get('GroupUserSubscription')).getStartActivity().getSheetId()
            if sheetDict.__contains__('RoleGroupSubscription'):
                userSyncDict['ROLE_GROUP_SHEET_METADATA_ID'] = sheetDict.get('RoleGroupSubscription')
            if sheetDict.__contains__('RoleSubscription'):
                userSyncDict['USER_ROLE_SHEET_METADATA_ID'] = sheetDict.get('RoleSubscription')
                userSyncDict['USER_ROLE_STATUS_VALUE']=applicationObj.getSheetMetadataById(sheetDict.get('RoleSubscription')).getStartActivity().getSheetId()
            workFlowRootbranch = applicationObj.getSheetMetadataById(sheetDict.get('User')).getWorkFlowRoot()
            actList = workFlowRootbranch.getActivityDefinitionList()
            userActId = actList[0].getSheetId()
            if len(actList)>2:
                userActId=workFlowRootbranch.getActivityDefinitionList()[1].getSheetId()
                if actList[1].getNodeType()=="Workflow" and actList[1].getAssignedFor()=="User":
                    from orangescape.model.engine.execution.ProcessUtil import getAssignedUser
                    userAssignedto = getAssignedUser(self.guestsession, userins, actList[1])
                else:
                    userAssignedto = actList[1].getAssignedTo()
            userSyncDict['USER_STATUS_VALUE']=userActId
            userSyncDict['USER_ASSIGNEDTO']= userAssignedto
            userSyncDict['DOMAIN']=getCompanyId(self.session.getUser().getValue('UserId'))
            userSyncDict['COMPANY_NAME']= userSyncDict['DOMAIN'].split('.')[0]
            usersync(userSyncDict).start()
        else:
            logging.error('Cannot perform user sync, kindly update the OAUTH_CONSUMER_KEY and OAUTH_CONSUMER_SECRET in config file')

    def schemaSync(self, metadatatype=None, applicationid=None):

        params = self.getQueryStringParams(self.request.query_string)

        if params.get('userid'):
            from flow.model.server.migration.FlowVersioning import AbstractVersionHandler
            AbstractVersionHandler(self.getSession()).doVersioning(params)
            logging.info('Tables created successfully')
            self.__commit__()
        else:
            from orangescape.model.server.sql import engine
            query_string = self.request.query_string
            params = self.getQueryStringParams(query_string)
            if params.get('createdb'):
                sql = 'CREATE DATABASE IF NOT EXISTS %s'%(config.SQL_SCHEMA)
                logging.info("Create DB SQL :%s"%(sql))
                from google.appengine.api import rdbms
                conn = rdbms.connect(instance = config.SQL_INSTANCE)
                cursor = conn.cursor()
                cursor.execute(sql)
                conn.close()
            else:
                from orangescape.model.server.sql.classes import Base
                Base.metadata.create_all(engine)
                logging.info('Tables created successfully')

    def dropTables(self, metadatatype=None, applicationid=None):

        from sqlalchemy.engine import create_engine
        from sqlalchemy.engine.url import URL

        from orangescape.model.server.sql.SchemaGenerator import SchemaGen
        from orangescape.model.server.Context import getApplication

        DATABASE_NAME = config.SQL_SCHEMA
        engine = create_engine(URL('mysql+googledialect', database=DATABASE_NAME, username='root', password='root'),instance=config.SQL_INSTANCE, paramstyle='format')
        logging.info('SQL Engine created successfully')
        schemaGenerator = SchemaGen(getApplication(), DATABASE_NAME, True)

        schemaGenerator.generate()
        logging.info('SQL Metadata generated')
        schemaGenerator.drop_tables(engine)
        logging.info('Tables dropped successfully')

    def backendStart(self):
        logging.info('Backend is ')

    def flowSetup(self, applicationid=None, metadatatype=None):
        from flow.services.SetUp import invoke
        from orangescape.model.engine.Component import Component

        params = self.request.params
        args = {}
        for key, value in params.iteritems():
            args[key] = value

        response  = invoke(Component(self.getSession()), self.getSession().getMetadata('Company'), args, None, None)
        logging.info(response)
        self.__commit__()
        self.response.out.write(self.getJsonForJsonp(response))

    def initApp(self, applicationid, metadatatype):
        from flow.services.AppInitService import invoke
        from orangescape.model.engine.Component import Component
        response  = invoke(Component(self.getSession()), self.getSession().getMetadata('Company'), {'method':'initialize'}, None, None)
#        self.response.out.write('Application initilized Successfully')
        jsonStr = '{"response":"Application initilized Successfully"}'
        self.response.out.write(jsonStr)


    def exportdata(self, metadatatype=None, applicationid=None):
        from flow.custom.process.reportexport import prepareExport
        from orangescape.model.engine.Component import Component
        component = Component(self.getSession())
        params = self.getQueryStringParams(self.request.query_string)
        sheetMetaDataId = params.get('Sheetmetadataid')
        instanceId = params.get('instanceId')
        userid = params.get('userid')
        self.setUserId(userid)
        prepareExport(component,sheetMetaDataId,instanceId)
        self.__commit__()

    def masterDataExport(self, metadatatype=None, applicationid=None):
        from flow.services.MasterExportService import prepareExport
        from orangescape.model.engine.Component import Component
        component = Component(self.getSession())
        params = self.getQueryStringParams(self.request.query_string)
        mcsheetdataid = params.get('mcsheetdataid')
        userid = params.get('userid')
        self.setUserId(userid)
        prepareExport(component,mcsheetdataid)
        self.__commit__()

    def masterDataImport(self, metadatatype=None, applicationid=None):
        from flow.services.commonservices import CommonService
        from orangescape.model.engine.Component import Component
        component = Component(self.getSession())
        params = self.getQueryStringParams(self.request.query_string)
        logging.info('params ==== %s' %params)
        self.setUserId(params['userid'])
        commonService = CommonService(component)
        commonService.syncTempDataFromTaskQueue(params)
        self.__commit__()

    def masterBatchImport(self, metadatatype=None, applicationid=None):
        from flow.services.commonservices import CommonService
        from orangescape.model.engine.Component import Component
        component = Component(self.getSession())
        params = self.getQueryStringParams(self.request.query_string)
        self.setUserId(params['userid'])
        commonService = CommonService(component)
        if params.has_key('isMaster'):
            commonService.batchSyncMasterData(params)
        else:
            commonService.batchSyncMasterModelData(params)
        self.__commit__()

    def reportExport(self, metadatatype=None, applicationid=None):
        params = self.getQueryStringParams(self.request.query_string)
        userid = params.get('userid')
        SheetmetadataId = params.get('SheetmetadataId')
        SearchdefId = params.get('SearchdefId')
        column = params.get('column')
        filter = params.get('filter')
        order = params.get('order')
        datetitle = params.get('datetitle')
        from flow.services.requestExport import prepareExport
        from orangescape.model.engine.Component import Component
        component = Component(self.getSession())
        self.setUserId(userid)
        prepareExport(component,SheetmetadataId,SearchdefId,column,filter,order,datetitle)

def getCompanyId(userid):
        #company id is got from appconfig if configured else companyid is got from user EmailId
        from appconfig import COMPANYID
        companyid=None
        if COMPANYID:
              return COMPANYID
        if companyid==None:
            if userid.find('@')>=0:
                return  userid[userid.index('@')+1:]
            else:
                raise NameError('Companyid not specified')