'''
Created on Jun 27, 2011

@author: Pravin
'''
import os
import pickle
import appconfig as config
import logging

import migrate.changeset
from migrate.changeset.constraint import ForeignKeyConstraint
from sqlalchemy import Table, Column, Integer, BLOB,String, MetaData, ForeignKey
from sqlalchemy.types import Date, FLOAT, DateTime,REAL

from orangescape.model.server.util.basicutil import __compress__,__decompress__
from orangescape.model.server.sql import engine
from orangescape.model.server.util.SqlUtil import PK_COL_LEN, TBL_PREFIX, SYS_PREFIX, TMP_PREFIX, getValidColumnName, getValidTableName, getValidSchemaName


class AbstractVersionHandler:

    applicationLevelMismatchDict = dict()
    modelLevelMismatchDict = dict()
    deployedDirpath = os.getcwd()
    deployedDirpath = os.path.abspath(os.path.join(os.path.dirname(__file__),'../../../application'))

    def __init__(self):
        self.schema = config.SQL_SCHEMA
        self.engine = engine
        self.__orm_metadata__ = MetaData(bind = self.engine)
        self.__orm_metadata__.reflect(bind = self.engine, schema = self.schema)

    def getapplicationMetadata(self):
        readApplicationMetadataFile = open(os.path.join(self.deployedDirpath,'Application.pck'),'rb')
        applicationMetadata = pickle.load(readApplicationMetadataFile)
        readApplicationMetadataFile.close()
        return applicationMetadata[applicationMetadata['root']]['Application.SheetMetadata']

    def getApplicationDiffList(self):
        appLevelMismatchDict = dict()
        ''' Retrives the metadata JSNOON structure by parsing the application PCK binary file.  '''
        applicationSheetMetadataList = self.getapplicationMetadata()
        ''' Loop through the application Sheet metadata and retrieves the sheet metadata which having a mismatch count '''
        for modelId in applicationSheetMetadataList:
            getModelDiffDict = self.getModelDiff(modelId, applicationSheetMetadataList[modelId])
            if getModelDiffDict:
                self.modelLevelMismatchDict[modelId] = getModelDiffDict
                mismatchCount= getModelDiffDict.length
                appLevelMismatchDict[modelId] = {
                                                "Sheet Name" : applicationSheetMetadataList[modelId],
                                                "Mismatch Count" : mismatchCount,
                                                "ModelMismatches" : getModelDiffDict
                                            }
                self.applicationLevelMismatchDict[modelId] = {
                                                "Sheet Name" : applicationSheetMetadataList[modelId],
                                                "Mismatch Count" : mismatchCount,
                                                "ModelMismatches" : getModelDiffDict
                                            }
        return appLevelMismatchDict

    ''' Checks if a specific Sheet Metadata has mismatch in any or all of its columns or foreign keys against the migrated version.
        @param : SheetMetadataId
        @returns : Dictionary ListOfMismatchColumns    '''
    def getModelDiff(self, modelId, modelName):
        if modelId:
            templateDict = []
            changesdict =  dict()

            ''' Unpickle the deployed sheetmetadata.pck file using unPickleSheetMetadataFile
                @arguments:
                    1. sheetMetadataId
                    2.file path.
                @return:
                    SheetMetadata JSNOON structure.
            '''
            newVersionSheetMetaData = self.unPickleSheetMetadataFile(modelId,self.deployedDirpath)
            tblVerMetadata = self.__orm_metadata__.tables.get(getValidSchemaName(self.schema) + '.'+getValidTableName('tbl'+modelName))
            if tblVerMetadata:
                ''' Get the list of public ,reference, system cell list from both old and new version '''
                newVersionPublicCellDict ,newVersionSystemCellDict = self.getPublicCellsOfModel(newVersionSheetMetaData)
                oldVersionPublicCellDict  = self.getPublicCellsOfTable(modelName)
                oldVersionSystemCellDict = self.getSystemCellsOfTable(modelName)
                ''' Get the list of private cell list from both old and new version '''
                newVersionPrivateCellDict = self.getPrivateCellsOfModel(newVersionSheetMetaData)
                oldVersionPrivateCellDict = self.getPrivateCellsOfTable(modelName)
                ''' Get the list of reference cell list from both old and new version '''

                ''' Check the added public cells in new version '''
                for publicCell in newVersionPublicCellDict:
                    if not oldVersionPublicCellDict.has_key(publicCell):
                        print 'New Column ' + newVersionPublicCellDict[publicCell].get("Cell Name") + ' is added'
                        templateDict.append('New Column ' + newVersionPublicCellDict[publicCell].get("Cell Name") + ' is added')

                for publicCell in oldVersionPublicCellDict:
                    ''' Check the missing public cells in new version from old version '''
                    if not newVersionPublicCellDict.has_key(publicCell):
                        print 'Column ' + oldVersionPublicCellDict[publicCell].get("Cell Name") + ' is deleted'
                        templateDict.append('Column ' + oldVersionPublicCellDict[publicCell].get("Cell Name") + ' is deleted')
                    else:
                        ''' Check the data type change of public cells in new version from old version '''
                        if not oldVersionPublicCellDict[publicCell].get("Data Type") == newVersionPublicCellDict[publicCell].get("Data Type"):
                            print "Data type of column " + oldVersionPublicCellDict[publicCell].get("Cell Name") + " is changed from "+oldVersionPublicCellDict[publicCell].get("Data Type") + " to "+newVersionPublicCellDict[publicCell].get("Data Type")
                            templateDict.append("Data type of column " + oldVersionPublicCellDict[publicCell].get("Cell Name") + " is changed from "+oldVersionPublicCellDict[publicCell].get("Data Type") + " to "+newVersionPublicCellDict[publicCell].get("Data Type"))

                        if oldVersionPublicCellDict[publicCell].get("Data Type") == newVersionPublicCellDict[publicCell].get("Data Type"):
                            if oldVersionPublicCellDict[publicCell].get("Text Length") or newVersionPublicCellDict[publicCell].get("Text Length"):
                                if not oldVersionPublicCellDict[publicCell].get("Text Length") == newVersionPublicCellDict[publicCell].get("Text Length"):
                                    print "Text Length for column" + oldVersionPublicCellDict[publicCell].get("Cell Name") + " is changed from " + oldVersionPublicCellDict[publicCell].get("Text Length") + " to "+newVersionPublicCellDict[publicCell].get("Text Length")
                                    templateDict.append("Text Length for column" + oldVersionPublicCellDict[publicCell].get("Cell Name") + " is changed from " + oldVersionPublicCellDict[publicCell].get("Text Length") + " to "+newVersionPublicCellDict[publicCell].get("Text Length"))

                for privateCell in oldVersionPrivateCellDict:
                    ''' Check the missing private cells in new version from old version '''
                    if not newVersionPrivateCellDict.has_key(privateCell):
                        print 'Column ' + oldVersionPrivateCellDict[privateCell].get("Cell Name") + ' is deleted'
                        templateDict.append('Column ' + oldVersionPrivateCellDict[privateCell].get("Cell Name") + ' is deleted')
                    else:
                        ''' Check the private cells data type change in new version from old version '''
                        if not oldVersionPrivateCellDict[privateCell].get("Data Type") == newVersionPrivateCellDict[privateCell].get("Data Type"):
                            print "Data type of column " + oldVersionPrivateCellDict[privateCell].get("Cell Name") + " is changed from " + oldVersionPrivateCellDict[privateCell].get("Data Type") + " to " + newVersionPrivateCellDict[privateCell].get("Data Type")
                            templateDict.append("Data type of column " + oldVersionPrivateCellDict[privateCell].get("Cell Name") + " is changed from " + oldVersionPrivateCellDict[privateCell].get("Data Type") + " to " + newVersionPrivateCellDict[privateCell].get("Data Type")) 

                        if oldVersionPrivateCellDict[privateCell].get("Data Type") == newVersionPrivateCellDict[privateCell].get("Data Type"):
                            if oldVersionPrivateCellDict[privateCell].get("Text Length") or newVersionPrivateCellDict[privateCell].get("Text Length"):
                                if not oldVersionPrivateCellDict[privateCell].get("Text Length") == newVersionPrivateCellDict[privateCell].get("Text Length"):
                                    print "Text Length for column" + oldVersionPrivateCellDict[privateCell].get("Cell Name") + " is changed from " + oldVersionPrivateCellDict[privateCell].get("Text Length") + " to "+newVersionPrivateCellDict[privateCell].get("Text Length")
                                    templateDict.append("Text Length for column" + oldVersionPrivateCellDict[privateCell].get("Cell Name") + " is changed from " + oldVersionPrivateCellDict[privateCell].get("Text Length") + " to "+newVersionPrivateCellDict[privateCell].get("Text Length"))

                    ''' Check the added private cells in new version '''
                for privateCell in newVersionPrivateCellDict:
                    if not oldVersionPrivateCellDict.has_key(privateCell):
                        print 'New Column ' + newVersionPrivateCellDict[privateCell].get("Cell Name") + ' is added'
                        templateDict.append('New Column ' + newVersionPrivateCellDict[privateCell].get("Cell Name") + ' is added')

                for systemCell in oldVersionSystemCellDict:
                    ''' Check the missing private cells in new version from old version '''
                    if not newVersionSystemCellDict.has_key(systemCell):
                        print 'Column ' + oldVersionSystemCellDict[systemCell].get("Cell Name") + ' is deleted'
                        templateDict.append('Column ' + oldVersionSystemCellDict[systemCell].get("Cell Name") + ' is deleted')

                for systemCell in newVersionSystemCellDict:
                    if not oldVersionSystemCellDict.has_key(systemCell):
                        print 'New column ' + newVersionSystemCellDict[systemCell].get("Cell Name") + ' is added'
                        templateDict.append('New column ' + newVersionSystemCellDict[systemCell].get("Cell Name") + ' is added')

                return templateDict

    def getSheetDiff(self, modelid):
#        logging.info('pritngin applevel mismatches dict')
        mismatchModelList = self.applicationLevelMismatchDict
#        logging.info(mismatchModelList)
#        logging.info('priting sheet diff')
#        logging.info(mismatchModelList[modelid].get('ModelMismatches'))
        return mismatchModelList[modelid].get('ModelMismatches')


    def getPublicCellsOfModel(self,sheetMetaData):
            ''' Get Vivek's code and use it intelligently.'''
            publicCellDict = dict()
            systemCellDict = dict()
            ''' This will fetch the all public cells exists in the specified Sheet Metadata '''

            sheetPublicCellList = sheetMetaData[sheetMetaData['root']]['SheetMetadata.CellMetadata']
            ''' Loop through the sheetCellMetadata to fetch the public cell names '''
            for publicCellId in sheetPublicCellList:
                publicCellMetadata = sheetMetaData[publicCellId]
                publicCellFormat = self.getCellDataType(sheetMetaData,publicCellId)
                publicCellName = publicCellMetadata.get('CellName')
                if publicCellMetadata.get('TextLength'):
                        textLength = str(int(publicCellMetadata.get('TextLength')))
                elif publicCellFormat == 'Text':
                    textLength = '255'
                else:
                    textLength = ''
                if publicCellMetadata.get('RowNumber') < 0 or publicCellMetadata.get('ColumnNumber') < 0:
                    systemCellDict[publicCellName] =  {
                                                    "Cell Name" : publicCellMetadata.get('CellName')
                                                    }
                else:
                    publicCellDict[publicCellName] = {
                                                    "Cell Name" : publicCellName,
                                                    "Data Type" : publicCellFormat,
                                                    "Text Length" : textLength
                                                   }
            return publicCellDict, systemCellDict

    def getPublicCellsOfTable(self,modelName):
            publicCellDict = dict()
            ''' This will fetch the all public cells exists in the specified Sheet Metadata '''
            sheetPublicCellList = self.__orm_metadata__.tables.get(getValidSchemaName(self.schema) + '.'+getValidTableName('tbl'+modelName))
            for column in sheetPublicCellList.c:
                columnName = column.name
                columnType = str(column.type)
                if columnType.find('(')>0:
                    columnLength = columnType[columnType.find('(')+1:-1]
                else:
                    columnLength = ''
                publicCellDict[columnName] = {
                                                "Cell Name" : columnName,
                                                "Data Type" : columnType,
                                                "Text Length" : columnLength
                                               }
            return publicCellDict

    def getSystemCellsOfTable(self,modelName):
            systemCellDict = dict()
            ''' This will fetch the all public cells exists in the specified Sheet Metadata '''
            sheetPublicCellList = self.__orm_metadata__.tables.get(getValidSchemaName(self.schema) + '.'+getValidTableName('sys'+modelName))
            for column in sheetPublicCellList.c:
                columnName = column.name
                systemCellDict[columnName] = {
                                                "Cell Name" : columnName
                                               }
            return systemCellDict

    def getPrivateCellsOfTable(self,modelName):
            privateCellDict = dict()
            ''' This will fetch the all public cells exists in the specified Sheet Metadata '''
            sheetPublicCellList = self.__orm_metadata__.tables.get(getValidSchemaName(self.schema) + '.'+getValidTableName('tmp'+modelName))
            for column in sheetPublicCellList.c:
                columnName = column.name
                columnType = str(column.type)
                if columnType.find('(')>0:
                    columnLength = columnType[columnType.find('(')+1:-1]
                else:
                    columnLength = ''
                privateCellDict[columnName] = {
                                                "Cell Name" : columnName,
                                                "Data Type" : columnType,
                                                "Text Length" : columnLength
                                               }
            return privateCellDict
            
            
    def getPrivateCellsOfModel(self,sheetMetaData):
        privateCellDict = dict()
        rowMetadata = sheetMetaData[sheetMetaData['root']]['SheetMetadata.RowMetadata']
        ''' Loop through the rowMetadata and fetch the private cell names,
            while fetching,check whether the cell has a data structure and it has the sheetmetadata i.e parent ref,
            if so then it is a public cell else it is private .
        '''
        for rowMetadataId in rowMetadata:
            rowCellList = sheetMetaData[rowMetadataId]['RowMetadata.CellMetadata']
            for privateCellId in rowCellList:
                if sheetMetaData.get(privateCellId):
                    if not sheetMetaData[privateCellId].get('SheetMetadata'):
                        privateCellFormat = self.getCellDataType(sheetMetaData,privateCellId)
                        if sheetMetaData[privateCellId].get('TextLength'):
                                textLength = str(int(sheetMetaData[privateCellId].get('TextLength')))
                        elif sheetMetaData[privateCellId] == 'Text':
                            textLength = '255'
                        else:
                            textLength = ''
                        privateCellDict[privateCellId] = {
#                                                          "Cell Name" : rowCellList[privateCellId],
                                                          "Cell Name" : privateCellId,
                                                          "Data Type" : privateCellFormat,
                                                          "Text Length" : textLength
                                                         }
        return privateCellDict


    def getCellDataType(self,deployedVersionSheetMetaData,cellId):
        publicCellMetadata = deployedVersionSheetMetaData[cellId]
        if publicCellMetadata.get('FinalCategory') == "Category001" or publicCellMetadata.get('FinalCategory') == "Category010":
            return "Text"
        elif publicCellMetadata.get('FinalCategory') == "Category005" or publicCellMetadata.get('CellName') == "CreatedAt" or publicCellMetadata.get('CellName') == "ModifiedAt":
            return "DateTime"
        else:
            return "Double"