import logging
import csv
#logging.getLogger('sqlalchemy.engine').setLevel(logging.DEBUG)
import datetime
import appconfig as config

from orangescape.model.server.util.SqlUtil import getManyToOneName, getPKName, getValidTableName, getSysTableName, getTblTableName, getTmpTableName
from orangescape.model.server.sql import getMappedClass, Session
from orangescape.model.server.AbstractSession import AbstractSession
from orangescape.model.server.wrapper.Model import DynamicModel
from orangescape.model.server.db.DbError import MappingError, DB00001, DB00002

from sqlalchemy import or_, and_

MAX = 'max'
MIN = 'min'
SUM = 'sum'
AVG = 'avg'
COUNT = 'count'
oper_map = {'=':'__eq__', '>':'__gt__', '<':'__lt__', '>=':'__ge__', '<=':'__le__', '!=':'__ne__', 'like':'like', 'IN':'in_', 'in':'in_', 'or':or_, 'and':and_}

class SQLSession(AbstractSession):

    def __init__(self, application, cache=None):
        self.__sql_session__ = Session()
        self.readCache = {}
        self.one2many_cache={}
        AbstractSession.__init__(self, application)
        self.cache=cache

    def getMappedClass(self, metadata):
        return getMappedClass(metadata)

    def getSysMappedClass(self, metadata):
        return getSysMappedClass(metadata)

    def createDataObject(self, metadata, keyname):
        metadata_class = self.getMappedClass(metadata)
        pk = keyname or self.__generate_pk__(metadata)
        dataObj = metadata_class(pk)
        dataObj.setSQLSession(self)
        return dataObj

    def readFromDatastore(self, sheetId, metadata):
        if not sheetId:
            return None
        metadata_class = self.getMappedClass(metadata)
        entity = self.memcacheGet(sheetId)
        if not entity:
#            metadata_class = self.getMappedClass(metadata)
            entity=None
            from sqlalchemy.exc import ResourceClosedError
            try:
                st = datetime.datetime.now()
#                logging.info('SQLSession :: Read from ds for %s %s'%(sheetId, metadata.getSheetName()))
                entity = self.__sql_session__.query(metadata_class).get(sheetId)
                if not entity:
                    return None
                self.memcacheSet(sheetId, entity)
            except (ResourceClosedError, IndexError), e:
                logging.error('Error while reading data for the model %s with the id %s. Error %s'%(metadata.getSheetName(), sheetId, e))
#        else:
#            logging.info('READ:: entity %s(%s)exists in cache'%(metadata.getSheetName(), sheetId))
        if entity:
            entity.setSQLSession(self)
        return entity

    def getUser(self):
        return self.user

    def getQueryCacheKey(self, foreignKeyValue, referenceCell):
        return ''.join([foreignKeyValue, '.', referenceCell.getSheetId(), '.', referenceCell.getRowMetadata().getSheetMetadata().getSheetId()])

    def removeFromCache(self, id):
        if self.one2many_cache.has_key(id):
            del self.one2many_cache[id]

    def getListFromDatastore(self, model, referenceCell):
        cacheKey = self.getQueryCacheKey(model.getSheetId(), referenceCell)
        if self.one2many_cache.has_key(cacheKey):
#            logging.info('Getlist from cache for %s %s'%(model.getSheetId(), referenceCell.getSheetMetadata().getSheetName()))
            return self.one2many_cache[cacheKey]
#        logging.info('Getlist from ds for %s %s'%(model.getSheetId(), referenceCell.getRowMetadata().getSheetMetadata().getSheetName()))
        childMetadata = referenceCell.getRowMetadata().getSheetMetadata()
        childList={}
        baseQuery = {(referenceCell.getSheetId(), '='):model.getSheetId()}
        for sessionChild in self.queryFromDataStore(childMetadata, baseQuery, 5000):
            childList[sessionChild.getSheetId()] = sessionChild
        self.one2many_cache[cacheKey] = childList
        return childList

    def clearOne2ManyCache(self, model, referenceCell):
        cacheKey = self.getQueryCacheKey(model.getSheetId(), referenceCell)
        if self.one2many_cache.has_key(cacheKey):
            logging.info('clearOne2ManyCache %s %s'%(cacheKey, self.one2many_cache[cacheKey]))
            del self.one2many_cache[cacheKey]

    def applyFilter(self, criteria, metadata, baseQuery, orderBy=None, isDescending=False):
        metadata_class = self.getMappedClass(metadata)
        for f in baseQuery:
            if f[1]=='=':
                criteria = criteria.filter(getattr(metadata_class,f[0])==(baseQuery[f] or None))
            elif f[1]=='>':
                criteria = criteria.filter(getattr(metadata_class,f[0])>(baseQuery[f] or self.getNullValue(metadata.getCellMetadataById(f[0]))))
            elif f[1]=='<':
                criteria = criteria.filter(getattr(metadata_class,f[0])<(baseQuery[f] or self.getNullValue(metadata.getCellMetadataById(f[0]))))
            elif f[1]=='<=':
                criteria = criteria.filter(getattr(metadata_class,f[0])<=(baseQuery[f] or self.getNullValue(metadata.getCellMetadataById(f[0]))))
            elif f[1]=='>=':
                criteria = criteria.filter(getattr(metadata_class,f[0])>=(baseQuery[f] or self.getNullValue(metadata.getCellMetadataById(f[0]))))
            elif f[1]=='!=':
                criteria = criteria.filter(getattr(metadata_class,f[0])!=baseQuery[f])
            elif f[1]=='like':
                criteria = criteria.filter(getattr(metadata_class,f[0]).like(self.getNullCheckedString(baseQuery[f])))
            elif f[1]=='in':
                criteria = self.addInFilter(criteria, getattr(metadata_class,f[0]), baseQuery[f])
        if orderBy:
            if isDescending:
                criteria = criteria.order_by(getattr(metadata_class, orderBy).desc())
            else:
                criteria = criteria.order_by(getattr(metadata_class, orderBy))
        return criteria

    def addInFilter(self, criteria, lhs, values):
        if len(values)>0:
            return criteria.filter(lhs.in_(values))
        else:
            return criteria.filter(lhs.in_(['need to be removed']))
            self.__logger.info('the size of value list for IN filter is 0. Ignoring it from the query criteria.')

    def getNullCheckedString(self, oldValue):
        if oldValue:
            return '%'+str(oldValue)+'%'
        else:
            return '%'

    def prepare_result_set(self, rs):
        return map(lambda e:(self.readCache.get(e.getSheetId()) or e).setSQLSession(self), rs)

    def queryFromDataStore(self, metadata, baseQuery, limit=None, offset=0, orderBy=None, isDescending=False):
        metadata_class = self.getMappedClass(metadata)
        self.attach_by_metadataId(metadata.getSheetId())
        criteria = self.__sql_session__.query(metadata_class)
#        criteria =  self.applyAdvFilter(metadata, metadata_class, criteria, baseQuery, orderBy, isDescending)
        criteria = self.applyFilter(criteria, metadata, baseQuery, orderBy, isDescending)
        if limit is None:
            limit=1000
        queryResultlist=criteria[offset:offset+limit]
        self.detach_all()
#        logging.info('Query from ds for %s %s'%(metadata.getSheetName(), baseQuery))
        return self.prepare_result_set(queryResultlist)

    def applyAdvFilter(self, metadata, metadata_class, criteria, baseQuery, orderBy=None, isDescending=False):
        if baseQuery:
            f = self.constructFilter(metadata, metadata_class, baseQuery)
            if str(f): criteria = criteria.filter(f)
        if orderBy:
            if isDescending:
                criteria = criteria.order_by(getattr(metadata_class, orderBy).desc())
            else:
                criteria = criteria.order_by(getattr(metadata_class, orderBy))
        logging.debug('Query: %s'%(criteria))
        return criteria

    def constructFilter(self, metadata, metadata_class, baseQuery):
        bool_operator, filter_params = baseQuery
        f = list()
        for tup in filter_params:
            if len(tup)==2:
                nested = self.constructFilter(metadata, metadata_class, tup)
                if str(nested): f.append(nested)
            else:
                lhs, operator, rhs = tup
                if operator.lower()=='like':
                    if metadata.getCellMetadataById(lhs).getFinalCategory() not in ('Category010', 'String'):
                        import math
                        rhs = math.floor(rhs)
                        f.append(self.constructFilter(metadata, metadata_class, ('and', [(lhs, '>=', rhs), (lhs, '<', rhs+1)])))
                    else:
                        rhs = self.getNullCheckedString(rhs)
                        f.append(getattr(getattr(metadata_class, lhs), oper_map[operator])(rhs))
                else:
                    if rhs is None and operator in ('>', '>=', '<', '<='):
                        rhs = self.getNullValue(metadata.getCellMetadataById(lhs))
                    f.append(getattr(getattr(metadata_class, lhs), oper_map[operator])(rhs))
        return oper_map[bool_operator.lower()](*tuple(f))

    def getNullValue(self, cellMetadata):
        return '' if cellMetadata.isStr() else 0

    def adv_query_from_ds(self, metadata, baseQuery, limit=None, offset=0, orderBy=None, isDescending=False):
        logging.info('Executing advquery for model %s basequery %s orderBy %s isDescending %s, offset :%s, limit :%s'%(metadata.getSheetName(), str(baseQuery), orderBy, isDescending, offset, limit))
        metadata_class = self.getMappedClass(metadata)
        criteria = self.__sql_session__.query(metadata_class)
        criteria = self.applyAdvFilter(metadata, metadata_class, criteria, baseQuery, orderBy, isDescending)
        if limit is None: limit = 1000
        queryResultlist = criteria[offset:offset+limit]
        self.detach_all()
        return self.prepare_result_set(queryResultlist)

    def report_query(self, metadata, limit, baseQuery, properties, orderBy = None, isDescending = False, offset=0, inMemorySorting=False):
#        logging.info('Report query from ds for %s %s'%(metadata.getSheetName(), baseQuery))
        metadata_class = self.getMappedClass(metadata)

        logging.info('Executing report_query for model %s basequery %s orderBy %s isDescending %s properties :%s, offset :%s, limit :%s'%(metadata.getSheetName(), str(baseQuery), orderBy, isDescending, properties, offset, limit))
        if config.env != config.SL and logging.getLogger().level == logging.DEBUG:
            logging.debug('%s report_query :%s'%(metadata.getSheetName(), baseQuery))
            logging.debug('properties :%s'%(properties))

        columnList = map(lambda x: getattr(metadata_class, metadata.getPublicCellMetadata(x).getSheetId()), properties)
        criteria = self.__sql_session__.query(*(columnList))
        criteria = self.applyAdvFilter(metadata, metadata_class, criteria, baseQuery, orderBy, isDescending)
        if limit is None:
            limit=1000
        rs = criteria[offset:offset+limit]
        rows = map(lambda tup:list(tup), rs)
        return rows

    def checkWithMemcache(self, metadataIdDict, dbResults = None):
        if not metadataIdDict:
            return dict()
        cachedEntities = self.memcacheGetMulti(metadataIdDict.keys())
        for id in metadataIdDict:
            entity = cachedEntities.get(id)
            if entity:
                entity.setSQLSession(self)
            else:
                cachedEntities[id] = self.readFromDatastore(id, self.getApplication().getSheetMetadataById(metadataIdDict[id]))
        return cachedEntities

    def getMultiFromDatastore(self, metadataIdDict):
        logging.info('GetMulti from ds length %s'%(len(metadataIdDict)))
        return self.checkWithMemcache(metadataIdDict)

    def count(self, metadata, baseQuery):
        metadata_class = self.getMappedClass(metadata)
        criteria = self.__sql_session__.query(getattr(metadata_class, getPKName(metadata)))
#        criteria = self.applyFilter(criteria, metadata, baseQuery)
        criteria = self.applyAdvFilter(metadata, metadata_class, criteria, baseQuery)
        return criteria.count()

    def __get_projections__(self, metadata, metadataClass, measures, dimensions, operations):
        from sqlalchemy.sql.expression import func
        manyToOneCells = {}
        groupByList = []
        measureList = []
        outerJoinList = {}

        def get_manyToOne_property(cellName):
            refCellName, fetchCellName = cellName.split('.')

            if manyToOneCells.has_key(refCellName):
                refCell, manyToOneMetadata = manyToOneCells[refCellName]
            else:
                refCell = metadata.getPublicCellMetadata(refCellName)
                manyToOneMetadata = refCell.getReferredSheetMetadata()
                manyToOneCells[refCellName] = (refCell, manyToOneMetadata)
                outerJoinList[refCellName] = getattr(metadataClass, refCell.getSheetId())
#                outerJoinList[refCellName] = getattr(metadataClass, getManyToOneName(refCell))

            manyToOneClass = self.getMappedClass(manyToOneMetadata)
            fetchCell = manyToOneMetadata.getPublicCellMetadata(fetchCellName)
            return getattr(manyToOneClass, fetchCell.getSheetId())

        for i in xrange(len(dimensions)):
            cellName = dimensions[i]
            if cellName.find('.')>0:
                groupByList.append(get_manyToOne_property(cellName))
            else:
                cell = metadata.getPublicCellMetadata(cellName)
                if cell.isReferenceCell():
                    manyToOneMetadata = cell.getReferredSheetMetadata()
                    manyToOneClass = self.getMappedClass(manyToOneMetadata)
                    outerJoinList[cellName] = getattr(metadataClass, cell.getSheetId())
#                    outerJoinList[cellName] = getattr(metadataClass, getManyToOneName(cell))
                    groupByList.append(getattr(manyToOneClass, getPKName(manyToOneMetadata)))
                else:
                    groupByList.append(getattr(metadataClass, cell.getSheetId()))

        execute_operations = {MIN:lambda cellid:func.min(cellid), MAX:lambda cellid:func.max(cellid),
                              SUM:lambda cellid:func.sum(cellid), AVG:lambda cellid:func.avg(cellid),
                              COUNT:lambda cellid:func.count(cellid)
                              }

        for i in xrange(len(measures)):
            operation = operations[i]
            cellName = measures[i]
            if cellName.find('.')>0:
                measureList.append(execute_operations[operation](get_manyToOne_property(cellName)))
            else:
                cell = metadata.getPublicCellMetadata(cellName)
                if cell.isReferenceCell():
                    manyToOneMetadata = cell.getReferredSheetMetadata()
                    manyToOneClass = self.getMappedClass(manyToOneMetadata)
                    outerJoinList[cellName] = getattr(metadataClass, cell.getSheetId())
                    measureList.append(execute_operations[operation](getattr(manyToOneClass, getPKName(manyToOneMetadata))))
                else:
                    measureList.append(execute_operations[operation](getattr(metadataClass, cell.getSheetId())))

        return tuple(groupByList), tuple(measureList), outerJoinList

    def aggregate_query(self, metadata, limit, baseQuery, measures, dimensions, operations, orderBy = None, isDescending = False, offset=0):
        from sqlalchemy import func
        if limit is None:
            limit = 2000
        metadata_class = self.getMappedClass(metadata)
        groupByArgs, measuresArgs, outerJoins = self.__get_projections__(metadata, metadata_class, measures, dimensions, operations)

        logging.info('Executing aggregate_query for model %s basequery %s orderBy %s isDescending %s measures :%s, dimensions :%s, operations :%s, offset %s, limit %s'%(metadata.getSheetName(), str(baseQuery), orderBy, isDescending, measures, dimensions, operations, offset, limit))
        if config.env != config.SL and logging.getLogger().level == logging.DEBUG:
            logging.debug('%s aggregate_query :%s'%(metadata.getSheetName(), baseQuery))
            logging.debug('measures :%s'%(measures))
            logging.debug('dimensions :%s'%(dimensions))
            logging.debug('operations :%s'%(operations))

        criteria = self.__sql_session__.query(*(groupByArgs+measuresArgs))

#        criteria = self.applyFilter(criteria, metadata, baseQuery, orderBy, isDescending)
        criteria = self.applyAdvFilter(metadata, metadata_class, criteria, baseQuery, orderBy, isDescending)
#        criteria = criteria.outerjoin(*tuple(outerJoins.values()))
        for refCellName in outerJoins:
            refCell = metadata.getPublicCellMetadata(refCellName)
            pClass = self.getMappedClass(refCell.getReferredSheetMetadata())
            criteria = criteria.filter(getattr(metadata_class, refCell.getSheetId()) == getattr(pClass, getPKName(refCell.getReferredSheetMetadata())))

        rs = criteria.group_by(*groupByArgs)[offset:offset+limit]
        rows = map(lambda tup:list(tup), rs)
        return rows

    def clearCache(self):
        self.one2many_cache.clear()
        AbstractSession.clearCache(self)

    def commit_datastore(self):

        deletedIds = {}
        for modelDict in self.deletedSheets.values():
            for id, model in modelDict.iteritems():
                deletedIds[id] = id
                self.__sql_session__.delete(model.getDataObject())

        self.commitAttachments()
        dataDict = dict()
        for metadataId, instanceDict in self.updatedSheets.iteritems():
            for id, instance in instanceDict.iteritems():
                instance.getDataObject().setSQLSession(None)
                dataDict[id] = instance.getDataObject()
                self.attach(instance.getDataObject())
        try:
            self.__sql_session__.commit()
            self.memcacheSetMulti(dataDict)
            self.memcacheDeleteMulti(deletedIds)
            self.clearCache()
        except Exception, e:
            logging.error('Error while commit %s. Rolling back transaction'%(e))
            self.clearCache()
            self.__sql_session__.rollback()
            raise

    def close(self):
        self.clearCache()
        self.__sql_session__.close()

    def detach(self, entity):
        self.__sql_session__.expunge(entity)

    def attach(self, entity):
        self.__sql_session__.add(entity)

    def detach_all(self):
        self.__sql_session__.expunge_all()

    def attach_all(self):
        for metadataId in self.updatedSheets:
            self.attach_by_metadataId(metadataId)

    def attach_by_metadataId(self, metadataId):
        dirtyEntities = self.updatedSheets.get(metadataId)
        if dirtyEntities:
            for instanceId, instance in dirtyEntities.iteritems():
                self.attach(instance.getDataObject())

    def getORMMetadata(self):
        return __orm_metadata__

    def getAttachmentClass(self):
        from orangescape.model.server.sql import getAttachmentClass
        return getAttachmentClass()

    def createAttachment(self, keyName):
        return self.getAttachmentClass()(id = keyName)

    def getAttachmentRecord(self, id):
        return self.__sql_session__.query(self.getAttachmentClass()).get(id)

    def commitAttachments(self):
        for id, content in self.attachmentCache['updated'].iteritems():
            attachment = self.getAttachmentRecord(id)
            if attachment is None:
                attachment = self.getAttachmentClass()(id = id)
            attachment.content=content
            self.__sql_session__.add(attachment)

        for id in self.attachmentCache['deleted'].keys():
            attachment = self.getAttachmentRecord(id)
            if attachment:
                self.__sql_session__.delete(attachment)

    def getAttachmentFromDS(self, id):
        attachment = self.getAttachmentRecord(id)
        if attachment:
            return attachment.content
        return None

    def memcacheGet(self, key):
        data = self.readCache.get(key)
        if data is None:
            if self.cache is not None:
                data = self.cache.get(key)
                if data is not None:
                    self.readCache[key] = data
        return data

    def memcacheSet(self, key, value):
        self.detach(value)
        self.readCache[key] = value
        if self.cache is not None:
            return self.cache.set(key, value, time=config.CACHE_EXPIRATION)

    def memcacheGetMulti(self, keyList):
        dataDict = self.getDictMulti(keyList)
        if len(dataDict) > 0:
            if len(dataDict) == len(keyList):
                return dataDict
            elif self.cache is not None:
                tempList = [sheetId for sheetId in keyList if not dataDict.__contains__(sheetId)]
                cacheData = self.cache.get_multi(tempList)
                for k, v in cacheData.iteritems():
                    self.readCache[k] = v
                    dataDict[k] = v
        elif self.cache is not None:
            dataDict = self.cache.get_multi(keyList)
            self.setDictMulti(dataDict)
        return dataDict

    def memcacheSetMulti(self, mapping):
        self.setDictMulti(mapping)
        if self.cache is not None:
            failedKeys = self.cache.set_multi(mapping, time=config.CACHE_EXPIRATION)
            if failedKeys:
                logging.error('Failed to update in memcache keys %s out of %s. %s'%(len(mapping), len(failedKeys), failedKeys))
                faildData = {}
                for key in failedKeys:
                    faildData[key] = mapping.get(key)
                while not self.cache.delete_multi(faildData):
                    logging.warn('memcache Delete multi failed')
                for key, entity in faildData.iteritems():
                    if not self.cache.set(key, entity, time=config.CACHE_EXPIRATION):
                        logging.error('Re-try failed for updating memcache for the id %s'%(key))

    def memcacheDeleteMulti(self, mapping):
        self.deleteDictMulti(mapping.keys())
        if self.cache is not None:
            return self.cache.delete_multi(mapping)

    def getDictMulti(self, keyList):
        localDict = dict()
        for key in keyList:
            obj = self.readCache.get(key)
            if obj is not None:
                localDict[key]=obj
        return localDict

    def setDictMulti(self, mapping):
        for key,val in mapping.iteritems():
            self.readCache[key]=val

    def deleteDictMulti(self, keyList):
        for key in keyList:
            if self.readCache.__contains__(key):
                del self.readCache[key]

    def clearSecondLevelCache(self):
        self.cache.flush_all()

    def put_entities(self, entities):
        for entity in entities:
            self.attach(entity)

    def put_attachments(self, attachmentList):
        for attachment in attachmentList:
            self.attach(attachment)

    def raw_csv_import(self, metadata, csv_data, colList=None):
        if isinstance(csv_data, list):
            dt = csv_data
            if not colList:
                if csv_data:
                    colList = dt[0].keys()
                else:
                    colList = []
        else:
            csvString = csv_data#.encode('utf-8')
            s = csv.StringIO(csvString)
            dt = csv.DictReader(s, delimiter=",")

            if dt.fieldnames is None:
                reader = csv.reader(s, delimiter=',')
                dt.fieldnames = reader.next()
            colList = dt.fieldnames

        self.__insert_or_update__(metadata, dt, colList)

    def __resolve_csv_columns__(self, metadata, csvFieldList):
        cells = dict([(cell.getCellName().upper(), cell) for cell in metadata.getCellMetadataList()])
        pk_name = getPKName(metadata)
        sys_cells = [pk_name, 'type_']
        tbl_cells = [pk_name, 'type_']
        for csv_col in csvFieldList:
            col_up = csv_col.upper()
            if cells.has_key(col_up):
                if cells[col_up].isSystemCell():
                    sys_cells.append(cells[col_up].getCellName())
                else:
                    tbl_cells.append(cells[col_up].getCellName())
            else:
                raise Exception('Column no %s "%s" in the csv file not found in the model "%s"'%(csvFieldList.index(csv_col)+1, csv_col, metadata.getSheetName()))
        return sys_cells, tbl_cells

    def __generate_sqls__(self, metadata, iterableData, colList):
        from orangescape.model.engine.execution.DateandTime import NOW
        from cStringIO import StringIO
        pk_name = getPKName(metadata)
        sys_default_val = {'SheetMetadataName' : metadata.getSheetMetadataName(), 'CreatedBy' : str(self.getUser().getSheetId()),
                'ModifiedBy' : str(self.getUser().getSheetId()), 'CreatedAt' :str(datetime.datetime.now()), 'ModifiedAt' : str(datetime.datetime.now()),
                'Tenant':str(self.getUser().getValue('Company')), 'Metadata': 'M_'+metadata.getSheetId(), 'type_':metadata.getSheetId(), 'AssignedTo':self.getUser().getValue('UserId').split('@')[0]}

        startActivity = metadata.getStartActivity()
        if startActivity:
            sys_default_val['Status'] = startActivity.getSheetId()

        sys_cells, tbl_cells = self.__resolve_csv_columns__(metadata, colList)
        sys_union = set(sys_cells) | set(sys_default_val.keys())

        tbl_buff = StringIO()
        sys_buff = StringIO()
        tmp_buff = StringIO()
        ids = {}

        tbl_buff.write('INSERT INTO %s (%s) VALUES (%s) '%(getValidTableName(getTblTableName(metadata)), ', '.join(tbl_cells), ', '.join(['%s' for i in xrange(len(tbl_cells))])))
        sys_buff.write('INSERT INTO %s (%s) VALUES (%s) '%(getValidTableName(getSysTableName(metadata)), ', '.join(sys_union), ', '.join(['%s' for i in xrange(len(sys_union))])))
        tmp_buff.write('INSERT INTO %s (%s) VALUES (%%s) '%(getValidTableName(getTmpTableName(metadata)), pk_name))

        cell_types = dict([(cell.getCellName(), cell.isStr()) for cell in metadata.getCellMetadataList()])

        def func(rec, colName):
            if colName in rec and rec[colName]:
                return rec[colName]
            return None

        tbl_params, sys_params, tmp_params = [], [], []
        for csv_record in iterableData:
            sys_cell_val = dict(sys_default_val)
            sys_cell_val.update(csv_record)
            sys_cell_val[pk_name] = sys_cell_val['SheetId']
            ids[sys_cell_val['SheetId']] = sys_cell_val['SheetId']

            tblValue = map(lambda colName:func(sys_cell_val, colName),tbl_cells)
            sysValue = map(lambda colName:func(sys_cell_val, colName),sys_union)

            tbl_params.append(tuple(tblValue))
            sys_params.append(tuple(sysValue))
            tmp_params.append((sys_cell_val['SheetId'], ))

        tbl_on_dup = ','.join(['%s=VALUES(%s)'%(colName, colName) for colName in tbl_cells if colName not in ('type_', pk_name, 'IsAdmin')])
        sys_on_dup = ','.join(['%s=VALUES(%s)'%(colName, colName) for colName in ['CreatedAt','ModifiedAt','ModifiedBy'] if colName not in ('type_', pk_name, 'SheetId')])
        tmp_on_dup = '%s=values(%s)'%(pk_name, pk_name)

        tbl_sql = "%s ON DUPLICATE KEY UPDATE %s"%(tbl_buff.getvalue(), tbl_on_dup)
        sys_sql = "%s ON DUPLICATE KEY UPDATE %s"%(sys_buff.getvalue(), sys_on_dup)
        tmp_sql = "%s ON DUPLICATE KEY UPDATE %s"%(tmp_buff.getvalue()[:-1], tmp_on_dup)

        return [(tbl_sql, tuple(tbl_params)), (sys_sql, tuple(sys_params)), (tmp_sql, tuple(tmp_params))], ids

    def __insert_or_update__(self, metadata, iterableData, colList):
        sql_params, ids = self.__generate_sqls__(metadata, iterableData, colList)

        for sql, params in sql_params:
            self.getConnection().execute(sql, *params)

        if hasattr(config, 'RAW_IMP_SQL') and config.RAW_IMP_SQL.has_key(metadata.getSheetName()):
            for sql in config.RAW_IMP_SQL[metadata.getSheetName()]:
                if logging.getLogger().level == logging.DEBUG or config.DEBUGGER_ENABLED == 1:
                    logging.debug('RawCSV Import SQL :%s'%(sql))
                self.getConnection().execute(sql)

        if metadata.getSheetName() in ('Company', 'User'):
            """Delete the Company record & User records from memcache since its directly updated in backend"""
            self.memcacheDeleteMulti(ids)

    def execute_sql(self, sql, *params):
        logging.info('executing direct sql %s %s'%(sql, str(params)))
        self.getConnection().execute(sql, *params)

    def getConnection(self):
        return self.__sql_session__.connection()

    def raw_table_join_query(self, metadata, baseQuery, columns, orderBy = None, isDescending = False, offset=0, limit=None):
        from cStringIO import StringIO
        selectColsList = []
        tTblAlias = "ttbl"
        sysTblAlias = "stbl"
        referredModelDict = {}
        leftOuterDict = {}
        innerJoinList = []
        tInnerJoin = [tTblAlias+".id="+sysTblAlias+".id"]
        fromTbl = getValidTableName(getSysTableName(metadata))
        fromTTblExist = False
        fromTblAlias = sysTblAlias
        for col in columns:
            cellObj = metadata.getPublicCellMetadata(col)
            if cellObj.isSystemCell() and not cellObj.isCascade():
                selectColsList.append(sysTblAlias+"."+col)
            elif cellObj.isReferenceCell():
                referredSheet = cellObj.getReferredSheetMetadata()
                fetchCell = cellObj.getSearchDefinitionList()[0].getDisplayCell()
                fetchCellName = fetchCell.getCellName()
                if cellObj.isCascade():
                    cascadeTblAlias = "tbl"+col
                    casacadeSysAlias = "sys"+col
                    innerJoinList.append((getValidTableName(getTblTableName(referredSheet)), cascadeTblAlias, sysTblAlias+"."+col+"="+cascadeTblAlias+".id"))
                    innerJoinList.append((getValidTableName(getSysTableName(referredSheet)), casacadeSysAlias, cascadeTblAlias+".id"+"="+casacadeSysAlias+".id"))
                    selectColsList.append(casacadeSysAlias+"."+fetchCellName+" as "+col)
                else:
                    if fetchCell.isSystemCell():
                        referredTblName = getValidTableName(getSysTableName(referredSheet))
                    else:
                        referredTblName = getValidTableName(getTblTableName(referredSheet))
                    referredTblAlias = col
                    referredModelDict[referredTblAlias] = referredTblName
                    leftOuterDict[referredTblAlias] = [tTblAlias+"."+col+"="+referredTblAlias+".id"]
                    selectColsList.append(referredTblAlias+"."+fetchCellName+" as "+col)
            else:
                fromTTblExist = True
                fromTblAlias = tTblAlias
                fromTbl = getValidTableName(getTblTableName(metadata))
                selectColsList.append(tTblAlias+"."+col)

        tbl_buff = StringIO()
        tbl_buff.write('SELECT %s FROM %s as %s '%(', '.join(selectColsList), fromTbl, fromTblAlias))

        if tInnerJoin and fromTTblExist:
            sysTable = getValidTableName(getSysTableName(metadata))
            tbl_buff.write('INNER JOIN %s as %s ON %s '%(sysTable, sysTblAlias, ', '.join(tInnerJoin)))
        for refModelAlias, referModel in referredModelDict.iteritems():
            if leftOuterDict.has_key(refModelAlias):
                tbl_buff.write('LEFT OUTER JOIN %s as %s ON %s '%(referModel, refModelAlias, ', '.join(leftOuterDict[refModelAlias])))

        for innerJoin in innerJoinList:
            table, alias, filters = innerJoin
            tbl_buff.write('INNER JOIN %s as %s ON %s '%(table, alias, filters))

        filters = self.constructJoinQueryFilter(baseQuery)
        if filters:
            tbl_buff.write('where %s '%filters)

        if orderBy:
            cellObj = metadata.getPublicCellMetadata(orderBy)
            if cellObj.isSystemCell():
                tbl_buff.write('ORDER BY %s.%s '%(sysTblAlias, orderBy))
            else:
                tbl_buff.write('ORDER BY %s.%s '%(tTblAlias, orderBy))
            if isDescending:
                tbl_buff.write(' DESC ')
            else:
                tbl_buff.write(' ASC ')

        if limit is None:
            limit = 1000
        tbl_buff.write('LIMIT %s '%limit)

        sql = tbl_buff.getvalue()
        sql = sql.replace('%','%%')
        logging.info('executing join query sql')
        logging.info(sql)
        rs = self.getConnection().execute(sql)
        return rs.fetchall()

    def constructJoinQueryFilter(self, base_join_query):
        val, filters = base_join_query
        from cStringIO import StringIO
        filterStr = StringIO()
        for filter in filters:
            if len(filter)==2:
                filterStr.write("( %s )"%(self.constructJoinQueryFilter(filter)))
                if not len(filters) == filters.index(filter)+1:
                    filterStr.write(" %s "%(val))
            else:
                lhs, operator, rhs = filter
                if operator == 'like':
                    rhs = '%%%s%%'%(rhs)
                if rhs is None:
                    filterStr.write("(%s %s null )"%(lhs, operator))
                else:
                    filterStr.write("(%s %s '%s' )"%(lhs, operator, rhs))
                if not len(filters) == filters.index(filter)+1:
                    filterStr.write(" %s "%(val))
        return filterStr.getvalue()

class TransientSession(SQLSession):

    TRANSIENT_KEY = 'transient'
    def __init__(self, application, session_id, is_commit, cache=None):
        SQLSession.__init__(self, application, cache)
        self.session_id = session_id
        self.is_commit = is_commit
        self.new_entities = {}

    def createDataObject(self, metadata, keyname):
        metadata_class = self.getMappedClass(metadata)
        pk = keyname or self.__generate_pk__(metadata)
        dataObj = metadata_class(pk)
        dataObj.setSQLSession(self)
        self.new_entities[pk] = metadata.getSheetId()
        logging.info('Creating transient dataobject Process : %s Id: %s. session_id :%s'%(metadata.getPublicName() or metadata.getSheetName(), pk, self.session_id))
        return dataObj

    def getListFromDatastore(self, model, referenceCell):
        cacheKey = self.getQueryCacheKey(model.getSheetId(), referenceCell)
        if self.one2many_cache.has_key(cacheKey):
#            logging.info('Getlist from cache for %s %s'%(model.getSheetId(), referenceCell.getSheetMetadata().getSheetName()))
            return self.one2many_cache[cacheKey]
#        logging.info('Getlist from memcache for %s %s'%(model.getSheetId(), referenceCell.getRowMetadata().getSheetMetadata().getSheetName()))
        idList = self.memcacheGet(cacheKey)
        if idList:
            dictRef = self.memcacheGetMulti(idList)
            rs = {}
            for id, entity in dictRef.iteritems():
                entity.setSQLSession(self)
                rs[id] = entity
            return rs
        #query from db
        childMetadata = referenceCell.getRowMetadata().getSheetMetadata()
        childList={}
        baseQuery = {(referenceCell.getSheetId(), '='):model.getSheetId()}
        for sessionChild in self.queryFromDataStore(childMetadata, baseQuery, 5000):
            childList[sessionChild.getSheetId()] = sessionChild
        self.one2many_cache[cacheKey] = childList
        logging.info('getListFromDatastore %s %s %s %s len %s'%(childMetadata.getPublicName() or childMetadata.getSheetName(), model.getSheetId(), referenceCell.getSheetMetadata().getSheetName(), referenceCell.getCellName(), len(childList)))
        return childList

    def commit_datastore(self):
        if self.is_commit:
            self.commit_into_db()
        else:
            self.commit_into_cache()

    def commit_into_cache(self):
        logging.info('Transient session commit...')
        deletedIds = {}
        for modelDict in self.deletedSheets.values():
            for id, model in modelDict.iteritems():
                deletedIds[id] = id
        dataDict = dict()
        one2many_cache = {}

        for metadataId, instanceDict in self.updatedSheets.iteritems():
            for id, instance in instanceDict.iteritems():
                for refCell in instance.getSheetMetadata().getManyToOneList():
#                    logging.info('Checking reference change instance %s (%s)- %s(%s) - %s'%(instance.getSheetMetadata().getPublicName(), instance.getSheetId(), refCell.getSheetId(), refCell.getCellName(), instance.get(refCell)))
                    oldData = self.memcacheGet(instance.getSheetId())
                    if self.isReferenceChanged(instance, oldData, refCell):
                        if instance.get(refCell) is not None:
                            qryCacheKey = self.getQueryCacheKey(instance.get(refCell), refCell)
                            idList = one2many_cache.get(qryCacheKey)
                            if idList is None:
                                idList = self.memcacheGet(qryCacheKey)
                                if idList is None:
                                    one2many_cache[qryCacheKey] = idList = []
                                else:
                                    one2many_cache[qryCacheKey] = idList
                            if instance.getSheetId() not in idList:
                                idList.append(instance.getSheetId())
                        if oldData and oldData.get(refCell) is not None:
                            qryCacheKey = self.getQueryCacheKey(oldData.get(refCell), refCell)
                            idList = one2many_cache.get(qryCacheKey) or self.memcacheGet(qryCacheKey)
                            if idList is not None and instance.getSheetId() in idList:
                                idList.remove(instance.getSheetId())
                instance.getDataObject().setSQLSession(None)
                dataDict[id] = instance.getDataObject()

        transient_entities = self.cache.get(self.TRANSIENT_KEY, namespace=self.session_id) or {}
        transient_entities.update(self.new_entities)
        self.cache.set(self.TRANSIENT_KEY, transient_entities, namespace=self.session_id)

        try:
            self.memcacheSetMulti(dataDict)
            self.memcacheDeleteMulti(deletedIds)
            self.memcacheSetMulti(one2many_cache)
            self.clearCache()
        except Exception, e:
            logging.error('Error while commiting Transient Session %s'%(e))
            self.clearCache()
            raise

    def attach(self, entity):
        if self.is_commit or entity.getSheetId() not in self.new_entities:
            logging.info('Attaching entity %s'%(entity.getSheetId()))
            self.__sql_session__.add(entity)
        else:
            logging.info('Transient entity with not commit, So not attaching %s'%(entity.getSheetId()))

    def commit_into_db(self):
        transient_entities = self.cache.get(self.TRANSIENT_KEY, namespace=self.session_id)
        if transient_entities:
            for id in transient_entities:
                metadata = self.getMetadataById(transient_entities[id])
                logging.info('Attaching the transient object to Session %s %s'%(metadata.getPublicName() or metadata.getSheetName(), id))
                instance = self.get(id, metadata)
                self.put(instance)
        SQLSession.commit_datastore(self)
        self.clear_transient_cache()

    def clear_transient_cache(self):
        transient_entities = self.cache.get(self.TRANSIENT_KEY, namespace=self.session_id)
        if transient_entities:
            logging.info('Clearing the transient entities %s from memcache %s'%(transient_entities.keys(), self.memcacheDeleteMulti(transient_entities)))
        logging.info('Clearing the transient cache %s'%(self.cache.delete(self.TRANSIENT_KEY, namespace=self.session_id)))

    def isReferenceChanged(self, model, oldData, refCell):
        oldRefValue = None
        if oldData:
            oldRefValue = oldData.get(refCell)
        if oldRefValue is None and model.get(refCell) is None:
            return False
        if oldRefValue is None and model.get(refCell) is not None:
            return True
        if model.get(refCell) is None and oldRefValue is not None:
            return True
        return model.get(refCell) != oldRefValue
