from json import write
from SerializeUtil import getDisplayCellName
import logging
from orangescape.model.server.PerfMonitor import startPerfMonitor, stopPerfMonitor
from orangescape.model.server.util.ConversionUtil import convertFromEngineFormat
from Serialize import Serializer

def jsonWrite(obj):
    return write(obj)

def getInboxJson(session, sheetMetadata , objectList, gridViewName, displayName=None, serializeChild=False, totalCount=None):
    pm = startPerfMonitor(sheetMetadata.getSheetName(),'serialize','serializeInbox')
    specifedSheetMetadataDict = session.getApplication().getFormMapping().get(sheetMetadata.getSheetName())
    specifiedGridViewDict = None
    additionalArgs = {"sheetMetadata": sheetMetadata, "displayName": displayName, "serializeChild": serializeChild, "totalCount": totalCount}
    if specifedSheetMetadataDict:
        specifiedGridViewDict = specifedSheetMetadataDict.get(gridViewName)
    #to support for backward compatibility,since via jquery api they would have queried on private model
    if specifiedGridViewDict is None:
        modifiedByCellId = sheetMetadata.getPublicCellMetadata("ModifiedBy").getSheetId()
        currentSheetName = sheetMetadata.getSheetName()
        specifiedGridViewDict = {'root': {'ManyToOne': {}, 'OneToMany': {modifiedByCellId+'.'+currentSheetName: {'ManyToOne': {}, 'OneToMany': {}}}}}
    inboxJsonStr = InboxSerialization(session, specifiedGridViewDict).getJson(objectList, additionalArgs)
    stopPerfMonitor(pm)
    return inboxJsonStr

def getInboxCsv(session, sheetMetadata, objectList, cellNameList=[]):
    csvStr = CsvExport(session).exportToCsv(sheetMetadata, objectList, cellNameList)
    return csvStr

class InboxSerialization(Serializer):
    def getJson(self, objList, args):
        #objList's sheetMetadata
        sheetMetadata = args.get("sheetMetadata")

        user = self.session.getUser()
        userId = user.getSheetId()
        #open the main brace
        self.ioAppend("{")
        self.ioAppend(jsonWrite("root"))
        self.ioAppend(":")
        self.ioAppend(jsonWrite(userId))
        if args.get("serializeChild"):
            self.serialize(user)
            userStrBuffer = self.processedDict.get(userId)
            userStrBuffer.append(",")
            userStrBuffer.append(self.getStaticOneToManyList(sheetMetadata, objList, args))
            if args.get("totalCount"):
                userStrBuffer.append(",")
                userStrBuffer.append(jsonWrite('TotalCount'))
                userStrBuffer.append(":")
                userStrBuffer.append(jsonWrite(args.get("totalCount")))

            userLoginId = user.getValue('UserLoginModel')
            if userLoginId:
                userLogin = self.session.get(userLoginId, self.session.getApplication().getSheetMetadata('UserLoginModel'))
                self.serialize(userLogin)

            #serialize the list of objects one by one
            modifiedByCellId = sheetMetadata.getPublicCellMetadata("ModifiedBy").getSheetId()
            currentSheetName = sheetMetadata.getSheetName()
            #logging.info(modifiedByCellId+"."+currentSheetName)
            specifiedDict = self.specifiedGridViewDict.get("root").get("OneToMany").get(modifiedByCellId+"."+currentSheetName)
            if specifiedDict:
                [self.serialize(obj, specifiedDict) for obj in objList  if obj.getSheetId() != userId]

            #append all the many to one reference, individual item loops will be closed inside processMultiDict
            self.processMultiDict()
        else:
            self.ioAppend(",")
            self.ioAppend(jsonWrite(userId))
            self.ioAppend(":{")
            self.ioAppend(self.getStaticOneToManyList(sheetMetadata, objList, args))
            self.ioAppend("}")

        #close the main brace
        self.ioAppend("}")
        return self.fileStr.getvalue()

    def getStaticOneToManyList(self, sheetMetadata, objList, args):
        strBuffer = list()
        keyName = "ModifiedBy." + sheetMetadata.getSheetName()
        strBuffer.append(jsonWrite(keyName))
        strBuffer.append(":{")
        childLen = len(objList)
        for obj in objList:
            childLen = childLen - 1
            cellName = 'SheetName'
            if args.get("displayName"):
                cellName = args.get("displayName")
            if args.get("serializeChild"):
                strBuffer.append(jsonWrite(obj.getSheetId()))
                strBuffer.append(":")
                strBuffer.append(jsonWrite(obj.getValue(cellName)))
            else:
                #for search dialog we are using aggregate query for performance improvement
                strBuffer.append(jsonWrite(obj[0]))
                strBuffer.append(":")
                strBuffer.append(jsonWrite(obj[1]))
            if childLen > 0:
                strBuffer.append(",")
        strBuffer.append("}")
        return ''.join(strBuffer)

class CsvExport(Serializer):
    def exportToCsv(self, sheetMetadata, objectList, cellNameList):
        cellMetadataList = list()
        publicCellMetadata = sheetMetadata.getPublicCellMetadataDict()

        if len(cellNameList)>0:
            for cellName in cellNameList:
                if publicCellMetadata.__contains__(cellName):
                    cellMetadataList.append(sheetMetadata.getPublicCellMetadata(cellName))
        else:
            cellMetadataList = sheetMetadata.getCellMetadataList()
        referenceCellMetadataList = list()
        newCellMetadataList = list()
        for cell in cellMetadataList:
            cellName=cell.getCellName()
            if cell.isReferenceCell():
                referenceCellMetadataList.append(cell)
            else:
                newCellMetadataList.append(cell)

        columnNameList = [cell.getCellName() for cell in newCellMetadataList]
        [columnNameList.append(refCell.getCellName()) for refCell in referenceCellMetadataList]
        self.ioAppend(','.join(columnNameList))
        self.ioAppend('\n')

        for obj in objectList:
            strBuffer = list()
            if isinstance(obj,list):
                self.ioAppend(','.join(['%s' % el for el in obj]))
                self.ioAppend('\n')
            else:
                self.serializePublicCellMetadata(strBuffer , obj, newCellMetadataList)
                self.serializeManyToOneReference(strBuffer, obj, referenceCellMetadataList)
                self.processedDict[obj.getSheetId()] = strBuffer


        manyToOneObjDict = self.session.getMulti(self.getMultiDict)
        #to get the same orderby as resutlset,we removed iterating from self.processedDict
        for obj in objectList:
            if not isinstance(obj,list):
                sheetId = obj.getSheetId()
                strBuffer = self.processedDict.get(sheetId)
                for manyToOneValue in self.manyToOneDict.get(sheetId):
                    if manyToOneValue == '':
                        strBuffer.append('')
                    else:
                        for manyToOneId, displayCellName in manyToOneValue.iteritems():
                            manyToOneObj = manyToOneObjDict.get(manyToOneId)
                            if manyToOneObj is not None:
                                #need to handle format for refcell display name
                                strBuffer.append(manyToOneObj.getValue(displayCellName))

                self.ioAppend(','.join(['%s' % el for el in strBuffer]))
                self.ioAppend('\n')

        return self.fileStr.getvalue()

    def serializePublicCellMetadata(self, strBuffer, obj, cellMetadataList):
        for cell in cellMetadataList:
            cellValue = obj.get(cell)
            category = cell.getValue('FinalCategory')
            formatStr = cell.getFinalFormat()
            formattedValue = convertFromEngineFormat(cellValue, category, formatStr)
            if isinstance(formattedValue, str) and formattedValue.find(",") > -1:
                formattedValue = "\""+formattedValue+"\""
            strBuffer.append(formattedValue)

    def serializeManyToOneReference(self, strBuffer, obj, referenceCellMetadataList):
        sheetId = obj.getSheetId()
        self.manyToOneDict[sheetId] = list()
        for refCell in referenceCellMetadataList:
            cellValue = obj.get(refCell)
            if cellValue is not None:
                refSheetMetadata = refCell.getReferredSheetMetadata()
                displayCellName = getDisplayCellName(refCell)
                refSheetMetadataId = refSheetMetadata.getSheetId()
                self.getMultiDict[cellValue] = refSheetMetadataId
                self.manyToOneDict[sheetId].append({cellValue:displayCellName})
            else:
                self.manyToOneDict[sheetId].append('')
