import logging
from orangescape.model.engine.engineconstants import OR_PROCESS, AND_PROCESS, REAL_OR_PROCESS
from appconfig import deployedat
import appconfig as config
from ServerUtil import geFinalEditorFormName, getFinalInboxFormName, _get_metadata_path
from orangescape.model.engine.execution.ProcessUtil import getContextBranch
import systemconfig

def invokeMetadataSync(session, params):
    from orangescape.model.engine.Component import Component
    component = Component(session)

    userid = params.get('userid')

    userMetadata = session.getApplication().getSheetMetadata('User')
    userIns = session.getIfExists(userid, userMetadata)
    if userIns:
        session.setUser(userIns)
        globalInst = session.getIfExists('global', session.getApplication().getSheetMetadata('Global'))
        if globalInst:
            sync = MetadataSync(component, globalInst, params)

            newSheetDict = sync.createObjects(globalInst)
            if params.get('migrate'):
                sync.migration()

            appRoleMetadata = session.getApplication().getSheetMetadata('AppRole')
            if newSheetDict.has_key(appRoleMetadata.getSheetId()):
                companyInst = session.get(userIns.getValue('Company'), session.getApplication().getSheetMetadata('Company'))
                newApproleDict = newSheetDict[appRoleMetadata.getSheetId()]
                for appRoleInst in newApproleDict.itervalues():
                    moveApproleToNextState(component, appRoleInst, companyInst)
        else:
            resp = "Couldn't find the Global instance to do metadata sync"
            logging.error(resp)
            return resp
    elif params.get('forcecreateuser'):
        from orangescape.model.server.UserAuthentication import UserAuthentication
        auth = UserAuthentication(session, userid)
        userId = auth.authenticateUser(True)
    else:
        resp = "Couldn't find the user %s to do metadata sync"%(userid)
        logging.error(resp)
        return resp

    return None

def moveApproleToNextState(component, appRoleInst, companyInst):
    workFlowRootbranch = appRoleInst.getSheetMetadata().getWorkFlowRoot()
    if workFlowRootbranch:
        actList = workFlowRootbranch.getActivityDefinitionList()
        if len(actList) > 2 :
            if actList[1].getAssignedFor() == "AppRole" and appRoleInst.getValue('Status') == actList[0].getSheetId():
                logging.info('moveApproleToNextState %s'%(appRoleInst.getSheetName()))
                component.submit(appRoleInst, actList[0])
                component.update(appRoleInst,{'Tenant': companyInst.getSheetId()})

class MetadataSync:
    def __init__(self, component, globalInst, params=None):
        self.component = component
        self.session = component.getSession()
        self.application = self.session.getApplication()
        self.metadataModel = self.application.getSheetMetadata('Metadata')
        self.activityModel = self.application.getSheetMetadata('Activity')
        self.appRoleModel =  self.application.getSheetMetadata('AppRole')
        self.userModel =  self.application.getSheetMetadata('User')
        self.roleSubscriptionModel =  self.application.getSheetMetadata('RoleSubscription')
        self.to_be_sync_metadata = dict()
        self.to_be_created_dict={}
        self.to_be_updated_dict={}
        self.to_be_deleted_dict={}
        self.getNewAppRoles(globalInst)
        self.syncMetadata(globalInst, params)

    def createObjects(self, globalInst):
        self.syncActivity()

        rsMetadata = self.session.getMetadata('RoleSubscription')

        rs_data = None
        if self.to_be_created_dict.has_key(rsMetadata.getSheetId()):
            rs_data = self.to_be_created_dict[rsMetadata.getSheetId()]
            del self.to_be_created_dict[rsMetadata.getSheetId()]
        newSheetDict = self.component.batch_update(self.to_be_created_dict, self.to_be_updated_dict, self.to_be_deleted_dict)
        if rs_data:
            newSheetDict = self.component.batch_update({rsMetadata.getSheetId():rs_data}, None, None)
        self.component.update(globalInst,{'DeployedAt':deployedat})
        return newSheetDict

    def __generate_metadata_instance_id__(self, metadataId):
        return 'M_'+metadataId

    def __get_metadata_id__(self, metadataInstanceId):
        return metadataInstanceId[2:]

    def getNewAppRoles(self, globalInst):
        #Fetching appRole
        #temp code to handle run in studio
        if config.env == config.SL :
            appRoleDict = self.application.approles
        elif config.pickle:
            from pickleUtil import unpickle
            application_path = _get_metadata_path()
            appRoleDict = unpickle(application_path, 'approles.pck')
        else:
            from orangescape.application.approles import approles as appRoleDict

        #creating sheetName List for instance
        existingAppRoles = self.session.getList(globalInst, self.appRoleModel.getPublicCellMetadata('Global'))
        appRoleSheetNameList= [ ins.getSheetName() for ins in existingAppRoles ]
        #fetching new roles to be created
        newRoleList = set(appRoleDict.keys()).difference(appRoleSheetNameList)
        newRolesCreationList= [{'SheetId':key, 'SheetName':key, 'Global':globalInst.getSheetId()} for key in newRoleList]
        self.to_be_created_dict[self.appRoleModel.getSheetId()]=newRolesCreationList

    def syncMetadata(self, globalInst, params=None):
        sysmodelExclusionList=['SheetMetadata','CellMetadata','RowMetadata','ColumnMetadata','Version','ActivityDefinition','SearchDefinition','GridView','GridRow','GridColumn','GridCell','Formula','Node','Branch','InputportMetadata','OutputportMetadata','Header','Hierarchy','Section','Style','Level','GridRowSpan','TableStyle','Criteria','Condition','LevelRowSpan','Button','ActivityPermission','Property','Range','RangeValue','CustomStyle','ShortCut']
        if params is not None and params.get('folderid'):
            folderId = params['folderid']
            folder = self.application.getObject(folderId)
            logging.info('folder metadata sync for the folder ==== %s'%folderId)
            metadataDict = folder.getSheetMetadataList()
        else:
            metadataDict = self.application.getSheetMetadataList()
        metadataMetadataId = self.application.getSheetMetadata('Metadata').getSheetId()
#        self.to_be_sync_metadata = dict([('M_'+metadataId, metadataMetadataId) for metadataId in metadataDict if metadataDict[metadataId] not in sysmodelExclusionList])
        for metadataId in metadataDict :
            model = self.application.getSheetMetadataById(metadataId)
            if self.application.getSheetId() != 'Application001' or metadataDict[metadataId] not in sysmodelExclusionList:
                self.to_be_sync_metadata['M_'+metadataId] = metadataMetadataId
        modelInsDict = self.session.getMulti(self.to_be_sync_metadata)
        to_be_updated_metadata_dict = dict()
        to_be_created_metadata_list = list()

        for sheetId in self.to_be_sync_metadata:
            model=self.application.getSheetMetadataById(self.__get_metadata_id__(sheetId))
            rootBranch = model.getWorkFlowRoot()
            activityDefinitionList = list()
            if rootBranch:
                activityDefinitionList = rootBranch.getActivityDefinitionList()

            if modelInsDict.get(sheetId):
                to_be_updated_value_dict = {}
                modelInst = modelInsDict[sheetId]
                if modelInst.getValue('ModelName')!=model.getValue('SheetName'):
                    to_be_updated_value_dict['ModelName'] = model.getValue('SheetName')
                if modelInst.getValue('MenuName')!=model.getValue('PublicName'):
                    to_be_updated_value_dict['MenuName'] = model.getValue('PublicName')
                if modelInst.getValue('Global')!= globalInst.getSheetId():
                    to_be_updated_value_dict['Global'] = globalInst.getSheetId()
                if modelInst.getValue('ModelId')!= model.getValue('SheetId'):
                    to_be_updated_value_dict['ModelId'] = model.getValue('SheetId')
                if rootBranch and model.isMonitorEnabled():
                    if modelInst.getValue('StartActivity')!= activityDefinitionList[0].getSheetId():
                        to_be_updated_value_dict['StartActivity'] = activityDefinitionList[0].getSheetId()
                    if modelInst.getValue('EndActivity')!= activityDefinitionList[1].getSheetId():
                        to_be_updated_value_dict['EndActivity'] = activityDefinitionList[-1].getSheetId()
                if modelInst.getValue('IsPublic')!= model.isPublic():
                    to_be_updated_value_dict['IsPublic'] = model.isPublic()
                if config.PLATFORM == 'runtime-kf' and model.isPublic():
                    appRoleId = 'Ar_'+modelInst.getValue('ModelId')+'OutofBoxReport'
                    appRole = self.session.getIfExists(appRoleId, self.appRoleModel)
                    rsExits = False
                    if appRole:
                        rsList = self.session.getList(appRole, self.roleSubscriptionModel.getPublicCellMetadata('AppRole'))
                        for rs in rsList:
                            if rs.getValue('AppRole') == appRoleId:
                                rsExits = True
                    if not rsExits:
                        logging.info('creating role sub for report ==%s'%appRoleId)
                        self.to_be_created_dict[self.roleSubscriptionModel.getSheetId()]=[{'PrimaryUser':self.session.getUser().getValue('SheetId'),'AppRole':'Ar_'+model.getValue('SheetId')+'OutofBoxReport'}]
                if to_be_updated_value_dict:
                    to_be_updated_metadata_dict[self.__generate_metadata_instance_id__(model.getSheetId())] = to_be_updated_value_dict
            else:
                to_be_created_value_dict={'Global' : globalInst.getSheetId()}
                to_be_created_value_dict['SheetId'] = self.__generate_metadata_instance_id__(model.getValue('SheetId'))
                to_be_created_value_dict['ModelId'] = model.getValue('SheetId')
                to_be_created_value_dict['MenuName'] = model.getValue('PublicName')
                to_be_created_value_dict['ModelName'] = model.getValue('SheetName')
                if config.PLATFORM == 'runtime-kf':
                    self.to_be_created_dict[self.roleSubscriptionModel.getSheetId()]=[{'PrimaryUser':self.session.getUser().getValue('SheetId'),'AppRole':'Ar_'+model.getValue('SheetId')+'OutofBoxReport'}]
                if model.isMonitorEnabled() and rootBranch:
                    to_be_created_value_dict['StartActivity'] = activityDefinitionList[0].getSheetId()
                    to_be_created_value_dict['EndActivity'] = activityDefinitionList[-1].getSheetId()

                to_be_created_value_dict['IsPublic'] = model.isPublic()
                to_be_created_metadata_list.append(to_be_created_value_dict)
        if to_be_updated_metadata_dict:
            self.to_be_updated_dict[self.metadataModel.getSheetId()]=to_be_updated_metadata_dict
        self.to_be_created_dict[self.metadataModel.getSheetId()]=to_be_created_metadata_list

    def syncActivity(self):
        for metadataInstanceId in self.to_be_sync_metadata:
            self.syncActivityInSheet(metadataInstanceId)#self.application.getSheetMetadataById(__get_metadata_id__(modelId)))

    def syncActivityInSheet(self, metadataInstanceId):
        model = self.application.getSheetMetadataById(self.__get_metadata_id__(metadataInstanceId))
        rootWFbranch = model.getWorkFlowRoot()
        if model.isMonitorEnabled() and rootWFbranch:
            rootActivityList =rootWFbranch.getActivityDefinitionList()
            activityDict_in_sheet = dict()
            activityInsDict = dict()
            for activity in rootActivityList:
                self.get_Activity_in_Sheet(activity,activityDict_in_sheet)

            metadataIns = self.session.getIfExists(metadataInstanceId, self.metadataModel)
            if metadataIns:
                activityInsDict = self.getDict(self.session.getList(metadataIns, self.activityModel.getPublicCellMetadata('Metadata')))
            #self.iterateActivity(activityInsDict, rootActivityList, metadataInstanceId, None)
            self.iterateActivity(activityInsDict, activityDict_in_sheet, metadataInstanceId)

    def iterateActivity(self, activityInsDict, activityDict, metadataInstanceId):
        to_be_created_activity_list=list()
        to_be_updated_activity_dict=dict()
        #andOrRealOrActList=list()
        for activity in activityDict.itervalues():
            self.__create_OR_Update_ActivityInst__(activityInsDict, activity, metadataInstanceId, to_be_created_activity_list, to_be_updated_activity_dict)

        '''Updating the deleted activites IsDeleted flag to 1 '''
        if config.PLATFORM == 'runtime-kf':
            self.handleDelActivities(activityInsDict, activityDict, to_be_updated_activity_dict)
        self.create_or_update_component(to_be_created_activity_list, to_be_updated_activity_dict)

    def handleDelActivities(self, activityInsDict, activityDict_in_sheet, to_be_updated_activity_dict):
        diffList = set(activityInsDict.keys()) - set(activityDict_in_sheet.keys())
        for activity in diffList:
            to_be_updated_activity_dict[activity] = {'IsDeleted':1}
            logging.info('deleted activity id===%s'%activity)

    def __create_OR_Update_ActivityInst__(self,activityInsDict, activity, metadataInstanceId, to_be_created_activity_list =None , to_be_updated_activity_dict=None):
        activitySyncParam = ['SheetId', 'SheetName', 'SequenceNo', 'GoTo', 'NodeType']
        if self.activityModel.getPublicCellMetadataDict().has_key('FinalBusinessStep'):
            activitySyncParam.append('FinalBusinessStep')
        activityInst = activityInsDict.get(activity.getSheetId())
        if activityInst:
            to_be_updated_activty_value = {}
            for param in activitySyncParam:
                value = activity.getValue(param)
                #instanceValue = activityInst.getValue(param)
                if param == 'GoTo' and value:
                    value = activity.getGoTo().getSheetId()
                if activityInst.getValue(param) != value:
                    to_be_updated_activty_value[param] = value
            parentActivity = getContextBranch(activity).getActivityDefinition()
            if parentActivity and activityInst.getValue('ParentActivity') != parentActivity.getSheetId():
                to_be_updated_activty_value['ParentActivity'] = parentActivity.getSheetId()
            finalEditorForm = geFinalEditorFormName(activity)
            if activityInst.getValue('EditorFormName') != finalEditorForm:
                to_be_updated_activty_value['EditorFormName'] = finalEditorForm
            finalInboxForm = getFinalInboxFormName(activity)
            if activityInst.getValue('InboxFormName') != finalInboxForm:
                to_be_updated_activty_value['InboxFormName'] = finalInboxForm

            if to_be_updated_activty_value:
                to_be_updated_activity_dict[activityInst.getSheetId()] = to_be_updated_activty_value
        else:
            to_be_created_activity_value={}
            for parameter in activitySyncParam:
                value = activity.getValue(parameter)
                if parameter == 'GoTo' and value:
                    value = activity.getGoTo().getSheetId()
                to_be_created_activity_value[parameter] =  value
            to_be_created_activity_value['ParentActivity'] = activity.getValue('ParentActivity')
            to_be_created_activity_value['Metadata'] = metadataInstanceId
            to_be_created_activity_value['EditorFormName'] = geFinalEditorFormName(activity)
            to_be_created_activity_value['InboxFormName'] = getFinalInboxFormName(activity)
            to_be_created_activity_list.append(to_be_created_activity_value)

        self.__create_OR_Update_RoleSubscriptionInst__(activity)

    def __create_OR_Update_RoleSubscriptionInst__(self, activity):
        # Assigned List Format ==> [{'fName.fName@domain.com':['fName', 'sName', IsGroup]}, {'fName.fName@domain.com':['fName', 'sName', IsGroup]}]
        if activity.getValue('AssignedTo') and activity.getValue('AssignedList'):
            realUserDict = dict()
            usersDict = eval(activity.getValue('AssignedList'))
            for dt in usersDict:
                if usersDict[dt][2] == 1: #Checking whether the email id is user or group
                    appRole = self.session.get(dt.split('@', 1)[0], self.appRoleModel)
                    usersList = self.session.getList(appRole, self.roleSubscriptionModel.getPublicCellMetadata('AppRole'))
                    #print dict([(usr.getValue('PrimaryUser'), True) for usr in usersList])
                    realUserDict.update(dict([(usr.getValue('PrimaryUser'), True) for usr in usersList]))
                else:
                    realUserDict.update({dt: True})
            realUserIdList = realUserDict.keys()

            appRole = self.session.getIfExists(activity.getAssignedTo(), self.appRoleModel)
            appRoleId = activity.getAssignedTo()
            toBeDeletedList = None
            if appRole: # If the 'Approle' exists already checking 'RolesubScription' users
                existingRoleSubscription = self.session.getList(appRole, self.roleSubscriptionModel.getPublicCellMetadata('AppRole'))
                existingRoleSubscription = dict([(roleSubscription.getValue('PrimaryUser'), roleSubscription.getSheetId()) for roleSubscription in existingRoleSubscription])
                toBeCreatedList =  list(set(realUserIdList) - set(existingRoleSubscription.keys()))
                toBeDeletedList =  list(set(existingRoleSubscription.keys()) - set(realUserIdList))
                toBeDeletedList = [existingRoleSubscription[item] for item in toBeDeletedList]
            else:
                toBeCreatedList = realUserIdList

            tempList = [{'PrimaryUser': tobeCreated, 'AppRole':appRoleId} for tobeCreated in toBeCreatedList]

            if tempList:
                self.to_be_created_dict.setdefault(self.roleSubscriptionModel.getSheetId(), list()).extend(tempList)
            if toBeDeletedList:
                self.to_be_deleted_dict.setdefault(self.roleSubscriptionModel.getSheetId(), list()).extend(toBeDeletedList)

    def get_Activity_in_Sheet(self, activity, activityDict_in_sheet):
        #activityDict_in_sheet[activity.getSheetId()]=metadataId
        activityDict_in_sheet[activity.getSheetId()] = activity
        if activity.getNodeType()==AND_PROCESS or activity.getNodeType()==OR_PROCESS or activity.getNodeType()==REAL_OR_PROCESS:
            for branch in activity.getBranchList():
                for act in branch.getActivityDefinitionList():
                    self.get_Activity_in_Sheet(act,activityDict_in_sheet)

    def create_or_update_component(self, to_be_created_activity_list, to_be_updated_activity_dict):
        self.to_be_created_dict.setdefault(self.activityModel.getSheetId(), list()).extend(to_be_created_activity_list)

        if to_be_updated_activity_dict:
            self.to_be_updated_dict.setdefault(self.activityModel.getSheetId(), dict()).update(to_be_updated_activity_dict)

    def getDict(self, actList):
        activityDict = {}
        for ins in actList:
            activityDict[ins.getSheetId()] = ins
        return activityDict

    def create_roleSub(self):
        appRoleMetadata = self.session.getMetadata('AppRole')
        getMultiIds = {}
        to_be_created_dict = {}
        dataList = []
        for modelName in self.application.getSheetMetadataDict():
            metadata = self.application.getSheetMetadata(modelName)
            if metadata.isPublic() and metadata.getValue('PublicName') not in ['Location','Department','DocAttachment','','ProcessInstance','MasterDependency','','Company','AppRole','DependencyDefinition','User','BusinessStep','MasterData','MasterCategory','Global']:
                appRoleId = 'Ar_'+metadata.getValue('SheetId')+'OutofBoxReport'
                appRole = self.session.getIfExists(appRoleId, self.appRoleModel)

                rsExits = False
                if appRole:
                    rsList = self.session.getList(appRole, self.roleSubscriptionModel.getPublicCellMetadata('AppRole'))
                    for rs in rsList:
                        if rs.getValue('AppRole') == appRoleId:
                            logging.info('role sub exisstts === %s'%rs.getValue('PrimaryUser'))
                            rsExits = True
                if not rsExits:
                    logging.info('approle exists==%s'%appRoleId)
                    dataList.append({'PrimaryUser':metadata.getValue('CreatedBy').keys()[0],'AppRole':appRoleId})
        to_be_created_dict[self.roleSubscriptionModel.getSheetId()]=dataList
        logging.info('role dict === %s'%to_be_created_dict)
        self.component.batch_update(to_be_created_dict ,{}, {})

    def migration(self):
        self.create_roleSub()