#!/usr/bin/env python
from google.appengine.api import urlfetch

def make_fetch_call(url, payload=None, method=urlfetch.GET, headers=None,
                    allow_truncated=False, follow_redirects=True, **kwargs):
  if headers is None:
    headers = {}
  for arg, value in kwargs.items():
    headers['x_' + arg] = value

  return OAuthURLFetchResult(urlfetch.fetch(url, payload, method, headers,
                           allow_truncated, follow_redirects))

_POSSIBLE_RESULT_ARGS = frozenset([
    'oauth_state',
    'oauth_approval_url',
    'oauth_error',
    'oauth_error_text',
])

class OAuthURLFetchResult(object):
  def __init__(self, url_fetch_result):
    self.url_fetch_result = url_fetch_result
    for attr in _POSSIBLE_RESULT_ARGS:
      name = 'x_' + attr
      if name in url_fetch_result.headers:
        self.__setattr__(attr, url_fetch_result.headers[name])
        del url_fetch_result.headers[name]
      else:
        self.__setattr__(attr, None)

  def getResult(self):
    return self.url_fetch_result.content

  def getError(self):
    if self.url_fetch_result.status_code==403 or self.url_fetch_result.status_code==401:
        return True
    else:
        return False

  def getHeaders(self):
    return self.url_fetch_result.headers

  def __str__(self):
    data = {
        'url_fetch_result.content': self.url_fetch_result.content,
        'url_fetch_result.status_code': self.url_fetch_result.status_code,
        'url_fetch_result.content_was_truncated':
        self.url_fetch_result.content_was_truncated,
        'url_fetch_result.headers': self.url_fetch_result.headers,
        'oauth_approval_url': self.oauth_approval_url,
        'oauth_state': self.oauth_state,
        'oauth_error': self.oauth_error,
        'oauth_error_text': self.oauth_error_text,
    }
    return data.__str__()
